from __future__ import print_function, unicode_literals
import getopt
import sys
import os
import logging
from subprocess import Popen, PIPE
from usfm_tools.support import loutRenderer, contextRenderer, htmlRenderer, singlehtmlRenderer, csvRenderer, \
    readerise, mdRenderer, asciiRenderer, usxRenderer, mediawikiPrinter


class UsfmTransform(object):

    savedCWD = ''
    __logger = logging.getLogger('usfm_tools')

    @staticmethod
    def runscriptold(c, prefix=''):
        UsfmTransform.__logger.info(prefix + ':: ' + c)
        pp = Popen(c, shell=True, stdout=PIPE, stderr=PIPE, stdin=PIPE)
        for ln in pp.stdout:
            UsfmTransform.__logger.info(prefix + ln[:-1])

    @staticmethod
    def runscript(c, prefix='', repeatFilter=''):
        UsfmTransform.__logger.info(prefix + ':: ' + c)
        pp = Popen([c], shell=True, stdout=PIPE, stderr=PIPE, stdin=PIPE)
        (result, stderrdata) = pp.communicate()
        UsfmTransform.__logger.info(result)
        UsfmTransform.__logger.info(stderrdata)
        if not repeatFilter == '' and not stderrdata.find(repeatFilter) == -1:
            UsfmTransform.runscript(c, prefix, repeatFilter)

    @staticmethod
    def setup():
        c = """
        cd support/thirdparty
        rm -rf context
        mkdir context
        cd context
        curl -o first-setup.sh http://minimals.contextgarden.net/setup/first-setup.sh
        sh ./first-setup.sh
        . ./tex/setuptex
        cd ..
        curl -o usfm2osis.pl http://crosswire.org/ftpmirror/pub/sword/utils/perl/usfm2osis.pl
        """
        UsfmTransform.runscript(c)

    @staticmethod
    def buildLout(usfmDir, builtDir, buildName):
        UsfmTransform.__logger.info('Building Lout...')

        # Prepare
        UsfmTransform.__logger.info('Clean working dir')
        UsfmTransform.runscript('rm "' + builtDir + '/working/lout/*"', '       ')

        # Convert to Lout
        UsfmTransform.__logger.info('Converting to Lout')
        UsfmTransform.ensureOutputDir(builtDir + '/working/lout')
        c = loutRenderer.LoutRenderer(usfmDir, builtDir + '/working/lout/' + buildName + '.lout')
        c.render()

        # Run Lout
        UsfmTransform.__logger.info('Copying support files')
        UsfmTransform.runscript('cp support/lout/oebbook working/lout', '       ')
        UsfmTransform.__logger.info('Running Lout')
        UsfmTransform.runscript('cd "' + builtDir + '/working/lout"; lout "./' + buildName + '.lout" > "' + buildName +
                                '.ps"', '       ',
                                repeatFilter='unresolved cross reference')
        UsfmTransform.__logger.info('Running ps2pdf')
        UsfmTransform.runscript(
            'cd "' + builtDir + '/working/lout"; ps2pdf -dDEVICEWIDTHPOINTS=432 -dDEVICEHEIGHTPOINTS=648 "' +
            buildName + '.ps" "' + buildName + '.pdf" ',
            '       ')
        UsfmTransform.__logger.info('Copying into builtDir')
        UsfmTransform.runscript('cp "' + builtDir + '/working/lout/' + buildName + '.pdf" "' + builtDir + '/' +
                                buildName + '.pdf" ', '       ')

    @staticmethod
    def buildConTeXt(usfmDir, builtDir, buildName):
        UsfmTransform.__logger.info('Building PDF via ConTeXt...')

        # Convert to ConTeXt
        UsfmTransform.__logger.info('Converting to ConTeXt...')
        # c = texise.TransformToContext()
        # c.setupAndRun(usfmDir, 'working/tex', buildName)
        UsfmTransform.ensureOutputDir(builtDir + '/working/tex')
        UsfmTransform.ensureOutputDir(builtDir + '/working/tex-working')
        c = contextRenderer.ConTeXtRenderer(usfmDir, builtDir + '/working/tex/bible.tex')
        c.render()

    @staticmethod
    def buildWeb(usfmDir, builtDir, buildName, oebFlag=False):
        # Convert to HTML
        UsfmTransform.__logger.info('Building Web HTML...')
        UsfmTransform.ensureOutputDir(builtDir + '/' + buildName + '_html')
        c = htmlRenderer.HTMLRenderer(usfmDir, builtDir + '/' + buildName + '_html', oebFlag)
        c.render()

    @staticmethod
    def buildSingleHtml(usfmDir, builtDir, buildName):
        # Convert to HTML
        UsfmTransform.__logger.info('Building Single Page HTML...')
        UsfmTransform.ensureOutputDir(builtDir)
        c = singlehtmlRenderer.SingleHTMLRenderer(usfmDir, builtDir + '/' + buildName + '.html')
        c.render()

    @staticmethod
    def buildCSV(usfmDir, builtDir, buildName):
        # Convert to CSV
        UsfmTransform.__logger.info('Building CSV...')
        UsfmTransform.ensureOutputDir(builtDir)
        c = csvRenderer.CSVRenderer(usfmDir, builtDir + '/' + buildName + '.csv')
        c.render()

    @staticmethod
    def buildReader(usfmDir, builtDir, buildName):
        # Convert to HTML for online reader
        UsfmTransform.__logger.info('Building for Reader...')
        UsfmTransform.ensureOutputDir(builtDir + 'en_oeb')
        c = readerise.TransformForReader()
        c.setupAndRun(usfmDir, builtDir + 'en_oeb')

    @staticmethod
    def buildMarkdown(usfmDir, builtDir, buildName):
        # Convert to Markdown for Pandoc
        UsfmTransform.__logger.info('Building for Markdown...')
        UsfmTransform.ensureOutputDir(builtDir)
        c = mdRenderer.MarkdownRenderer(usfmDir, builtDir + '/' + buildName + '.md')
        c.render()

    @staticmethod
    def buildASCII(usfmDir, builtDir, buildName):
        # Convert to ASCII
        UsfmTransform.__logger.info('Building for ASCII...')
        UsfmTransform.ensureOutputDir(builtDir)
        c = asciiRenderer.ASCIIRenderer(usfmDir, builtDir + '/' + buildName + '.txt')
        c.render()

    @staticmethod
    def buildUSX(usfmDir, builtDir, buildName, byBookFlag):
        # Convert to USX
        UsfmTransform.__logger.info('Building for USX...')
        UsfmTransform.ensureOutputDir(builtDir)
        c = usxRenderer.USXRenderer(usfmDir, builtDir + '/', buildName, byBookFlag)
        c.render()

    @staticmethod
    def buildMediawiki(usfmDir, builtDir, buildName):
        # Convert to MediaWiki format for Door43
        UsfmTransform.__logger.info('Building for Mediawiki...')
        # Check output directory
        UsfmTransform.ensureOutputDir(builtDir + '/mediawiki')
        mediawikiPrinter.Transform().setupAndRun(usfmDir, builtDir + '/mediawiki')

    @staticmethod
    def ensureOutputDir(dir):
        if not os.path.exists(dir):
            os.makedirs(dir)

    @staticmethod
    def saveCWD():
        global savedCWD
        savedCWD = os.getcwd()
        root_dir_of_tools = os.path.dirname(os.path.abspath(__file__))
        os.chdir(root_dir_of_tools)

    @staticmethod
    def restoreCWD(): os.chdir(savedCWD)

    @staticmethod
    def run(argv):
        UsfmTransform.saveCWD()
        oeb_flag = False
        by_book_flag = False
        targets = ''
        build_dir = ''
        usfm_dir = ''
        build_name = ''

        UsfmTransform.__logger.info('Starting Build.')
        try:
            opts, args = getopt.getopt(argv, "sht:u:b:n:o",
                                       ["setup", "help", "target=", "usfmDir=", "builtDir=", "name=", "oeb",
                                        "fileByBook"])
        except getopt.GetoptError:
            UsfmTransform.usage()
            sys.exit(2)
        for opt, arg in opts:
            if opt in ("-h", "--help"):
                return UsfmTransform.usage()
            elif opt in ("-s", "--setup"):
                return UsfmTransform.setup()
            elif opt in ("-t", "--target"):
                targets = arg
            elif opt in ("-u", "--usfmDir"):
                usfm_dir = arg
            elif opt in ("-b", "--builtDir"):
                build_dir = arg
            elif opt in ("-n", "--name"):
                build_name = arg
            elif opt in ("-o", "--oeb"):
                oeb_flag = True
            elif opt in ("-f", "--fileByBook"):
                by_book_flag = True
            else:
                UsfmTransform.usage()

        if targets == 'context':
            UsfmTransform.buildConTeXt(usfm_dir, build_dir, build_name)
        elif targets == 'html':
            UsfmTransform.buildWeb(usfm_dir, build_dir, build_name, oeb_flag)
        elif targets == 'singlehtml':
            UsfmTransform.buildSingleHtml(usfm_dir, build_dir, build_name)
        elif targets == 'md':
            UsfmTransform.buildMarkdown(usfm_dir, build_dir, build_name)
        elif targets == 'reader':
            UsfmTransform.buildReader(usfm_dir, build_dir, build_name)
        elif targets == 'mediawiki':
            UsfmTransform.buildMediawiki(usfm_dir, build_dir, build_name)
        elif targets == 'lout':
            UsfmTransform.buildLout(usfm_dir, build_dir, build_name)
        elif targets == 'csv':
            UsfmTransform.buildCSV(usfm_dir, build_dir, build_name)
        elif targets == 'ascii':
            UsfmTransform.buildASCII(usfm_dir, build_dir, build_name)
        elif targets == 'csv':
            UsfmTransform.buildCSV(usfm_dir, build_dir, build_name)
        elif targets == 'usx':
            if by_book_flag:
                build_name = u''
            UsfmTransform.buildUSX(usfm_dir, build_dir, build_name, by_book_flag)
        else:
            UsfmTransform.usage()

            UsfmTransform.__logger.info('Finished.')
        UsfmTransform.restoreCWD()

    @staticmethod
    def usage():
        print("""
            USFM-Tools
            ----------

            Build script.  See source for details.

            Setup:
            transform.py --setup

        """)


if __name__ == "__main__":
    UsfmTransform.run(sys.argv[1:])



# -*- coding: utf-8 -*-
#

from .abstractRenderer import AbstractRenderer
import codecs

#
#   Simplest renderer. Ignores everything except ascii text.
#

class MarkdownRenderer(AbstractRenderer):

    def __init__(self, inputDir, outputFilename):
        # Unset
        self.f = None  # output file stream
        # IO
        self.outputFilename = outputFilename
        self.inputDir = inputDir
        # Position
        self.currentC = 1
        self.book = u''

    def render(self):
        self.f = codecs.open(self.outputFilename, 'w', 'utf_8_sig')
        self.loadUSFM(self.inputDir)
        self.run()
        self.f.close()

    def writeLog(self, s):
        print(s)

    # Support

    def escape(self, s):
        return(s)

    def renderTEXT(self, token):    self.f.write(self.escape(token.value))

    def renderH(self, token):       self.book = token.getValue()
    def renderMT(self, token):      self.f.write(u'\n\n# ' + token.value.upper() + u'\n\n')
    def renderMT2(self, token):     self.f.write(u'\n\n## ' + token.value.upper() + u'\n\n')
    def renderMS(self, token):      self.f.write(u'\n\n' + token.value + u'\n' + (u'=' * len(token.value)) + u'\n\n')
    def renderMS2(self, token):     self.f.write(u'\n\n' + token.value + u'\n' + (u'-' * len(token.value)) + u'\n\n')
    def renderP(self, token):       self.f.write(u'\n\n')
    def renderB(self, token):       self.f.write(u'\n\n')
    def renderS(self, token):       self.f.write(u'\n\ \n')
    def renderS2(self, token):      self.f.write(u'\n\ \n')
    def renderC(self, token):       self.currentC = token.value; self.f.write(u'\n\n [' + self.book + u' ' + self.currentC + u' ] \n\n')
    def renderV(self, token):       self.f.write(u' [' + self.currentC + u':' + token.value + u'] ')
    def renderQ(self, token):       self.f.write(u'\n|  ')
    def renderQ1(self, token):      self.f.write(u'\n|  ')
    def renderQ2(self, token):      self.f.write(u'\n|    ')
    def renderQ3(self, token):      self.f.write(u'\n|      ')
    def renderNB(self, token):      self.f.write(u'\n|  ')
    def renderLI(self, token):      self.f.write(u'* ')
    def renderPBR(self, token):     self.f.write(u'\n')

    def renderBDS(self, token):     self.f.write( u'**')
    def renderBDE(self, token):     self.f.write( u'**')
    def renderBDITS(self, token):   pass
    def renderBDITE(self, token):   pass

    def renderFS(self,token):       self.f.write(u'^[')
    def renderFE(self,token):       self.f.write(u']')
    def renderFR(self, token):      self.f.write(self.escape(token.value))
    def renderFT(self, token):      self.f.write(self.escape(token.value))
    def renderFQ(self, token):      self.f.write(self.escape(token.value))
    def renderFP(self,token):       self.f.write(u'\n\n')

    def renderXS(self,token):       self.f.write(u'^[')
    def renderXE(self,token):       self.f.write(u']')
    def renderXO(self, token):      self.f.write(self.escape(token.value))
    def renderXT(self, token):      self.f.write(self.escape(token.value))

    def render_imt1(self, token):   self.f.write(u'\n\n## ' + token.value.upper() + u'\n\n')
    def render_imt2(self, token):   self.f.write(u'\n\n### ' + token.value.upper() + u'\n\n')
    def render_imt3(self, token):   self.f.write(u'\n\n#### ' + token.value.upper() + u'\n\n')




# -*- coding: utf-8 -*-
#

from __future__ import print_function, unicode_literals
import sys
from pyparsing import Word, OneOrMore, nums, Literal, White, Group, Suppress, NoMatch, Optional, \
    CharsNotIn, MatchFirst
import logging

__logger = logging.getLogger('usfm_tools')

def usfmToken(key):
    return Group(Suppress(backslash) + Literal(key) + Suppress(White()))


def usfmBackslashToken(key):
    return Group(Literal(key))


def usfmEndToken(key):
    return Group(Suppress(backslash) + Literal(key + "*"))


def usfmTokenValue(key, value):
    return Group(Suppress(backslash) + Literal(key) + Suppress(White()) + Optional(value))


def usfmTokenNumber(key):
    return Group(Suppress(backslash) + Literal(key) + Suppress(White()) + Word(nums + '-()') + Suppress(White()))


# define grammar
# phrase = Word( alphas + "-.,!? —–‘“”’;:()'\"[]/&%=*…{}" + nums )
phrase = CharsNotIn("\n\\")
backslash = Literal("\\")
plus = Literal("+")

textBlock = Group(Optional(NoMatch(), "text") + phrase)
unknown = Group(Optional(NoMatch(), "unknown") + Suppress(backslash) + CharsNotIn(u' \n\t\\'))
escape = usfmTokenValue("\\", phrase)

id_token = usfmTokenValue("id", phrase)
ide = usfmTokenValue("ide", phrase)
sts = usfmTokenValue("sts", phrase)
h = usfmTokenValue("h", phrase)

mt = usfmTokenValue("mt", phrase)
mt1 = usfmTokenValue("mt1", phrase)
mt2 = usfmTokenValue("mt2", phrase)
mt3 = usfmTokenValue("mt3", phrase)
ms = usfmTokenValue("ms", phrase)
ms1 = usfmTokenValue("ms1", phrase)
ms2 = usfmTokenValue("ms2", phrase)
mr = usfmTokenValue("mr", phrase)
s = usfmTokenValue("s", phrase)
s1 = usfmTokenValue("s1", phrase)
s2 = usfmTokenValue("s2", phrase)
s3 = usfmTokenValue("s3", phrase)
s4 = usfmTokenValue("s4", phrase)
s5 = usfmTokenValue("s5", phrase)
r = usfmTokenValue("r", phrase)
p = usfmToken("p")
pi = usfmToken("pi")
pi2 = usfmToken("pi2")
b = usfmToken("b")
c = usfmTokenNumber("c")
cas = usfmToken("ca")
cae = usfmEndToken("ca")
cl = usfmTokenValue("cl", phrase)
v = usfmTokenNumber("v")
wjs = usfmToken("wj")
wje = usfmEndToken("wj")
q = usfmToken("q")
q1 = usfmToken("q1")
q2 = usfmToken("q2")
q3 = usfmToken("q3")
q4 = usfmToken("q4")
qa = usfmTokenValue("qa", phrase)
qac = usfmToken("qac")
qace = usfmEndToken("qac")
qc = usfmToken("qc")
qm = usfmToken("qm")
qm1 = usfmToken("qm1")
qm2 = usfmToken("qm2")
qm3 = usfmToken("qm3")
qr = usfmTokenValue("qr", phrase)
qss = usfmToken("qs")
qse = usfmEndToken("qs")
qts = usfmToken("qt")
qte = usfmEndToken("qt")
nb = usfmToken("nb")
m = usfmToken("m")

# Footnotes
fs = usfmTokenValue("f", plus)
fr = usfmTokenValue("fr", phrase)
fre = usfmEndToken("fr")
fk = usfmTokenValue("fk", phrase)
ft = usfmTokenValue("ft", phrase)
fq = usfmTokenValue("fq", phrase)
fqa = usfmTokenValue("fqa", phrase)
fqae = usfmTokenValue("fqa*", phrase)
fqb = usfmTokenValue("fqb", phrase)
fe = usfmEndToken("f")
fp = usfmToken("fp")

# Cross References
xs = usfmTokenValue("x", plus)
xdcs = usfmToken("xdc")
xdce = usfmEndToken("xdc")
xo = usfmTokenValue("xo", phrase)
xt = usfmTokenValue("xt", phrase)
xe = usfmEndToken("x")

# Transliterated
tls = usfmToken("tl")
tle = usfmEndToken("tl")

# Transliterated
scs = usfmToken("sc")
sce = usfmEndToken("sc")

# Italics
ist = usfmToken("it")
ien = usfmEndToken("it")

# Bold
bds = usfmToken("bd")
bde = usfmEndToken("bd")
bdits = usfmToken("bdit")
bdite = usfmEndToken("bdit")

li = usfmToken("li")
li1 = usfmToken("li1")
li2 = usfmToken("li2")
li3 = usfmToken("li3")
li4 = usfmToken("li4")
d = usfmTokenValue("d", phrase)
sp = usfmToken("sp")
adds = usfmToken("add")
adde = usfmEndToken("add")
nds = usfmToken("nd")
nde = usfmEndToken("nd")
pbr = usfmBackslashToken("\\\\")
mi = usfmToken("mi")

# Comments
rem = usfmTokenValue("rem", phrase)

# Tables
tr = usfmToken("tr")
th1 = usfmToken("th1")
th2 = usfmToken("th2")
th3 = usfmToken("th3")
th4 = usfmToken("th4")
th5 = usfmToken("th5")
th6 = usfmToken("th6")
thr1 = usfmToken("thr1")
thr2 = usfmToken("thr2")
thr3 = usfmToken("thr3")
thr4 = usfmToken("thr4")
thr5 = usfmToken("thr5")
thr6 = usfmToken("thr6")
tc1 = usfmToken("tc1")
tc2 = usfmToken("tc2")
tc3 = usfmToken("tc3")
tc4 = usfmToken("tc4")
tc5 = usfmToken("tc5")
tc6 = usfmToken("tc6")
tcr1 = usfmToken("tcr1")
tcr2 = usfmToken("tcr2")
tcr3 = usfmToken("tcr3")
tcr4 = usfmToken("tcr4")
tcr5 = usfmToken("tcr5")
tcr6 = usfmToken("tcr6")

# Table of Contents
toc = usfmTokenValue("toc", phrase)
toc1 = usfmTokenValue("toc1", phrase)
toc2 = usfmTokenValue("toc2", phrase)
toc3 = usfmTokenValue("toc3", phrase)

# Introductory Materials
is1 = usfmTokenValue("is1", phrase) | usfmTokenValue("is", phrase)
ip = usfmToken("ip")
iot = usfmToken("iot")
io1 = usfmToken("io1") | usfmToken("io")
io2 = usfmToken("io2")
ior_s = usfmToken("ior")
ior_e = usfmEndToken("ior")
imt = usfmTokenValue("imt", phrase)
imt1 = usfmTokenValue("imt1", phrase)
imt2 = usfmTokenValue("imt2", phrase)
imt3 = usfmTokenValue("imt3", phrase)

# Quoted book title
bk_s = usfmToken("bk")
bk_e = usfmEndToken("bk")

element = MatchFirst([ide, id_token, sts, h, toc, toc1, toc2, toc3, mt, mt1, mt2, mt3,
                      ms,
                      ms1,
                      ms2,
                      mr,
                      s,
                      s1,
                      s2,
                      s3,
                      s4,
                      s5,
                      r,
                      pi2,
                      pi,
                      p,
                      mi,
                      b,
                      c,
                      cas,
                      cae,
                      cl,
                      v,
                      wjs,
                      wje,
                      nds,
                      nde,
                      q,
                      q1,
                      q2,
                      q3,
                      q4,
                      qa,
                      qac,
                      qace,
                      qc,
                      qm,
                      qm1,
                      qm2,
                      qm3,
                      qr,
                      qss,
                      qse,
                      qts,
                      qte,
                      nb,
                      m,
                      fs,
                      fr,
                      fre,
                      fk,
                      ft,
                      fq,
                      fqa,
                      fqae,
                      fqb,
                      fe,
                      fp,
                      xs,
                      xdcs,
                      xdce,
                      xo,
                      xt,
                      xe,
                      ist,
                      ien,
                      bds,
                      bde,
                      bdits,
                      bdite,
                      li,
                      li1,
                      li2,
                      li3,
                      li4,
                      d,
                      sp,
                      adds,
                      adde,
                      tls,
                      tle,
                      is1,
                      ip,
                      iot,
                      io1,
                      io2,
                      ior_s,
                      ior_e,
                      imt,
                      imt1,
                      imt2,
                      imt3,
                      bk_s,
                      bk_e,
                      scs,
                      sce,
                      pbr,
                      rem,
                      tr,
                      th1,
                      th2,
                      th3,
                      th4,
                      th5,
                      th6,
                      thr1,
                      thr2,
                      thr3,
                      thr4,
                      thr5,
                      thr6,
                      tc1,
                      tc2,
                      tc3,
                      tc4,
                      tc5,
                      tc6,
                      tcr1,
                      tcr2,
                      tcr3,
                      tcr4,
                      tcr5,
                      tcr6,
                      textBlock,
                      escape,
                      unknown])

usfm = OneOrMore(element)


# input string
def parseString(unicodeString):
    try:
        cleaned = clean(unicodeString)
        tokens = usfm.parseString(cleaned, parseAll=True)
    except Exception as e:
        __logger.error(e)
        __logger.error(repr(unicodeString[:50]))
        sys.exit()
    return [createToken(t) for t in tokens]


def clean(unicodeString):
    # We need to clean the input a bit. For a start, until
    # we work out what to do, non breaking spaces will be ignored
    # ie 0xa0
    ret_value = unicodeString.replace('\xa0', ' ')

    # escape illegal USFM sequences
    ret_value = ret_value.replace('\\ ',  '\\\\ ')
    ret_value = ret_value.replace('\\\n', '\\\\\n')
    ret_value = ret_value.replace('\\\r', '\\\\\r')
    ret_value = ret_value.replace('\\\t', '\\\\\t')

    # check edge case if backslash is at end of line
    l = len(ret_value)
    if (l > 0) and (ret_value[l-1] == '\\'):
        ret_value += '\\' # escape it

    return ret_value


def createToken(t):
    options = {
        'id': IDToken,
        'ide': IDEToken,
        'sts': STSToken,
        'h': HToken,
        'mt': MTToken,
        'mt1': MTToken,
        'mt2': MT2Token,
        'mt3': MT3Token,
        'ms': MSToken,
        'ms1': MSToken,
        'ms2': MS2Token,
        'mr': MRToken,
        'p': PToken,
        'pi': PIToken,
        'pi2': PI2Token,
        'b': BToken,
        's': SToken,
        's1': SToken,
        's2': S2Token,
        's3': S3Token,
        's4': S4Token,
        's5': S5Token,
        'mi': MIToken,
        'r': RToken,
        'c': CToken,
        'ca': CASToken,
        'ca*': CAEToken,
        'cl': CLToken,
        'v': VToken,
        'wj': WJSToken,
        'wj*': WJEToken,
        'q': QToken,
        'q1': Q1Token,
        'q2': Q2Token,
        'q3': Q3Token,
        'q4': Q4Token,
        'qa': QAToken,
        'qac': QACToken,
        'qac*': QACEToken,
        'qc': QCToken,
        'qm': QMToken,
        'qm1': QM1Token,
        'qm2': QM2Token,
        'qm3': QM3Token,
        'qr': QRToken,
        'qs': QSSToken,
        'qs*': QSEToken,
        'qt': QTSToken,
        'qt*': QTEToken,
        'nb': NBToken,
        'f': FSToken,
        'fr': FRToken,
        'fr*': FREToken,
        'fk': FKToken,
        'ft': FTToken,
        'fq': FQToken,
        'fqa': FQAToken,
        'fqae': FQAEToken,
        'fqa*': FQAEToken,
        'fqb': FQAEToken,
        'f*': FEToken,
        'fp': FPToken,
        'x': XSToken,
        'xdc': XDCSToken,
        'xdc*': XDCEToken,
        'xo': XOToken,
        'xt': XTToken,
        'x*': XEToken,
        'it': ISToken,
        'it*': IEToken,
        'bd': BDSToken,
        'bd*': BDEToken,
        'bdit': BDITSToken,
        'bdit*': BDITEToken,
        'li': LIToken,
        'li1': LI1Token,
        'li2': LI2Token,
        'li3': LI3Token,
        'li4': LI4Token,
        'd': DToken,
        'sp': SPToken,
        'i*': IEToken,
        'add': ADDSToken,
        'add*': ADDEToken,
        'nd': NDSToken,
        'nd*': NDEToken,
        'sc': SCSToken,
        'sc*': SCEToken,
        'm': MToken,
        'tl': TLSToken,
        'tl*': TLEToken,
        '\\\\': EscapedToken,
        'rem': REMToken,
        'tr': TRToken,
        'th1': TH1Token,
        'th2': TH2Token,
        'th3': TH3Token,
        'th4': TH4Token,
        'th5': TH5Token,
        'th6': TH6Token,
        'thr1': THR1Token,
        'thr2': THR2Token,
        'thr3': THR3Token,
        'thr4': THR4Token,
        'thr5': THR5Token,
        'thr6': THR6Token,
        'tc1': TC1Token,
        'tc2': TC2Token,
        'tc3': TC3Token,
        'tc4': TC4Token,
        'tc5': TC5Token,
        'tc6': TC6Token,
        'tcr1': TCR1Token,
        'tcr2': TCR2Token,
        'tcr3': TCR3Token,
        'tcr4': TCR4Token,
        'tcr5': TCR5Token,
        'tcr6': TCR6Token,
        'toc1': TOC1Token,
        'toc2': TOC2Token,
        'toc3': TOC3Token,
        'is': IS1_Token,
        'is1': IS1_Token,
        'imt': IMT1_Token,
        'imt1': IMT1_Token,
        'imt2': IMT2_Token,
        'imt3': IMT3_Token,
        'ip': IP_Token,
        'iot': IOT_Token,
        'io': IO1_Token,
        'io1': IO1_Token,
        'io2': IO2_Token,
        'ior': IOR_S_Token,
        'ior*': IOR_E_Token,
        'bk': BK_S_Token,
        'bk*': BK_E_Token,
        'text': TEXTToken,
        'unknown': UnknownToken
    }
    for opt_key, opt_val in options.items():
        if t[0] == opt_key:
            if len(t) == 1:
                return opt_val()
            else:
                return opt_val(t[1])
    raise Exception(t[0])


# noinspection PyMethodMayBeStatic
class UsfmToken(object):
    def __init__(self, value=""):
        self.value = value

    def getValue(self): return self.value

    def isUnknown(self): return False

    def isID(self): return False

    def isIDE(self): return False

    def isSTS(self): return False

    def isH(self): return False

    def isTOC1(self): return False

    def isTOC2(self): return False

    def isTOC3(self): return False

    def isMT(self): return False

    def isMT2(self): return False

    def isMT3(self): return False

    def isMS(self): return False

    def isMS2(self): return False

    def isMR(self): return False

    def isR(self): return False

    def isP(self): return False

    def isPI(self): return False

    def isPI2(self): return False

    def isS(self): return False

    def isS2(self): return False

    def isS3(self): return False

    def isS4(self): return False

    def isS5(self): return False

    def isMI(self): return False

    def isC(self): return False

    def isCAS(self): return False

    def isCAE(self): return False

    def isCL(self): return False

    def isV(self): return False

    def isWJS(self): return False

    def isWJE(self): return False

    def isTEXT(self): return False

    def isQ(self): return False

    def isQ1(self): return False

    def isQ2(self): return False

    def isQ3(self): return False

    def isQ4(self): return False

    def isQA(self): return False

    def isQAC(self): return False

    def isQACE(self): return False

    def isQC(self): return False

    def isQM(self): return False

    def isQM1(self): return False

    def isQM2(self): return False

    def isQM3(self): return False

    def isQR(self): return False

    def isQSS(self): return False

    def isQSE(self): return False

    def isQTS(self): return False

    def isQTE(self): return False

    def isNB(self): return False

    def isFS(self): return False

    def isFR(self): return False

    def isFRE(self): return False

    def isFK(self): return False

    def isFT(self): return False

    def isFQ(self): return False

    def isFQA(self): return False

    def isFQAE(self): return False

    def isFQB(self): return False

    def isFE(self): return False

    def isFP(self): return False

    def isXS(self): return False

    def isXDCS(self): return False

    def isXDCE(self): return False

    def isXO(self): return False

    def isXT(self): return False

    def isXE(self): return False

    def isIS(self): return False

    def isIE(self): return False

    def isSCS(self): return False

    def isSCE(self): return False

    def isLI(self): return False

    def isLI1(self): return False

    def isLI2(self): return False

    def isLI3(self): return False

    def isLI4(self): return False

    def isD(self): return False

    def isSP(self): return False

    def isADDS(self): return False

    def isADDE(self): return False

    def isNDS(self): return False

    def isNDE(self): return False

    def isTLS(self): return False

    def isTLE(self): return False

    def isBDS(self): return False

    def isBDE(self): return False

    def isBDITS(self): return False

    def isBDITE(self): return False

    def isPBR(self): return False

    def isM(self): return False

    def isREM(self): return False

    def isTR(self): return False

    def isTH1(self): return False

    def isTH2(self): return False

    def isTH3(self): return False

    def isTH4(self): return False

    def isTH5(self): return False

    def isTH6(self): return False

    def isTHR1(self): return False

    def isTHR2(self): return False

    def isTHR3(self): return False

    def isTHR4(self): return False

    def isTHR5(self): return False

    def isTHR6(self): return False

    def isTC1(self): return False

    def isTC2(self): return False

    def isTC3(self): return False

    def isTC4(self): return False

    def isTC5(self): return False

    def isTC6(self): return False

    def isTCR1(self): return False

    def isTCR2(self): return False

    def isTCR3(self): return False

    def isTCR4(self): return False

    def isTCR5(self): return False

    def isTCR6(self): return False

    def is_toc1(self): return False

    def is_toc2(self): return False

    def is_toc3(self): return False

    def is_is1(self): return False

    def is_imt1(self): return False

    def is_imt2(self): return False

    def is_imt3(self): return False

    def is_ip(self): return False

    def is_iot(self): return False

    def is_io1(self): return False

    def is_io2(self): return False

    def is_ior_s(self): return False

    def is_ior_e(self): return False

    def is_bk_s(self): return False

    def is_bk_e(self): return False


class UnknownToken(UsfmToken):
    def renderOn(self, printer):
        return printer.renderUnknown(self)

    def isUnknown(self): return True

class EscapedToken(UsfmToken):
    def renderOn(self, printer):
        self.value = '\\'
        return printer.renderTEXT(self)

    def isUnknown(self): return True


class IDToken(UsfmToken):
    def renderOn(self, printer):
        return printer.renderID(self)

    def isID(self): return True


class IDEToken(UsfmToken):
    def renderOn(self, printer):
        return printer.renderIDE(self)

    def isIDE(self): return True


class STSToken(UsfmToken):
    def renderOn(self, printer):
        return printer.renderSTS(self)

    def isSTS(self): return True


class HToken(UsfmToken):
    def renderOn(self, printer):
        return printer.renderH(self)

    def isH(self): return True


class TOC1Token(UsfmToken):
    def renderOn(self, printer):
        return printer.renderTOC1(self)

    def isTOC1(self): return True


class TOC2Token(UsfmToken):
    def renderOn(self, printer):
        return printer.renderTOC2(self)

    def isTOC2(self): return True


class TOC3Token(UsfmToken):
    def renderOn(self, printer):
        return printer.renderTOC3(self)

    def isTOC3(self): return True


class MTToken(UsfmToken):
    def renderOn(self, printer):
        return printer.renderMT(self)

    def isMT(self): return True


class MT2Token(UsfmToken):
    def renderOn(self, printer):
        return printer.renderMT2(self)

    def isMT2(self): return True


class MT3Token(UsfmToken):
    def renderOn(self, printer):
        return printer.renderMT3(self)

    def isMT3(self): return True


class MSToken(UsfmToken):
    def renderOn(self, printer):
        return printer.renderMS(self)

    def isMS(self): return True


class MS2Token(UsfmToken):
    def renderOn(self, printer):
        return printer.renderMS2(self)

    def isMS2(self): return True


class MRToken(UsfmToken):
    def renderOn(self, printer):
        return printer.renderMR(self)

    def isMR(self): return True


class MIToken(UsfmToken):
    def renderOn(self, printer):
        return printer.renderMI(self)

    def isMI(self): return True


class RToken(UsfmToken):
    def renderOn(self, printer):
        return printer.renderR(self)

    def isR(self): return True


class PToken(UsfmToken):
    def renderOn(self, printer):
        return printer.renderP(self)

    def isP(self): return True


# noinspection PyMethodMayBeStatic
class BToken(UsfmToken):
    def renderOn(self, printer):
        return printer.renderB(self)

    def isB(self): return True


class CToken(UsfmToken):
    def renderOn(self, printer):
        return printer.renderC(self)

    def isC(self): return True


class CASToken(UsfmToken):
    def renderOn(self, printer):
        return printer.renderCAS(self)

    def isCAS(self): return True


class CAEToken(UsfmToken):
    def renderOn(self, printer):
        return printer.renderCAE(self)

    def isCAE(self): return True


class CLToken(UsfmToken):
    def renderOn(self, printer):
        return printer.renderCL(self)

    def isCL(self): return True


class VToken(UsfmToken):
    def renderOn(self, printer):
        return printer.renderV(self)

    def isV(self): return True


class TEXTToken(UsfmToken):
    def renderOn(self, printer):
        return printer.renderTEXT(self)

    def isTEXT(self): return True


class WJSToken(UsfmToken):
    def renderOn(self, printer):
        return printer.renderWJS(self)

    def isWJS(self): return True


class WJEToken(UsfmToken):
    def renderOn(self, printer):
        return printer.renderWJE(self)

    def isWJE(self): return True


class SToken(UsfmToken):
    def renderOn(self, printer):
        return printer.renderS(self)

    def isS(self): return True


class S2Token(UsfmToken):
    def renderOn(self, printer):
        return printer.renderS2(self)

    def isS2(self): return True


class S3Token(UsfmToken):
    def renderOn(self, printer):
        return printer.renderS3(self)

    def isS3(self): return True


class S4Token(UsfmToken):
    def renderOn(self, printer):
        return printer.renderS4(self)

    def isS4(self): return True


class S5Token(UsfmToken):
    def renderOn(self, printer):
        return printer.renderS5(self)

    def isS5(self): return True


class QToken(UsfmToken):
    def renderOn(self, printer):
        return printer.renderQ(self)

    def isQ(self): return True


class Q1Token(UsfmToken):
    def renderOn(self, printer):
        return printer.renderQ1(self)

    def isQ1(self): return True


class Q2Token(UsfmToken):
    def renderOn(self, printer):
        return printer.renderQ2(self)

    def isQ2(self): return True


class Q3Token(UsfmToken):
    def renderOn(self, printer):
        return printer.renderQ3(self)

    def isQ3(self): return True


class Q4Token(UsfmToken):
    def renderOn(self, printer):
        return printer.renderQ4(self)

    def isQ4(self): return True


class QAToken(UsfmToken):
    def renderOn(self, printer):
        return printer.renderQA(self)

    def isQA(self): return True


class QACToken(UsfmToken):
    def renderOn(self, printer):
        return printer.renderQAC(self)

    def isQAC(self): return True


class QACEToken(UsfmToken):
    def renderOn(self, printer):
        return printer.renderQACE(self)

    def isQACE(self): return True


class QCToken(UsfmToken):
    def renderOn(self, printer):
        return printer.renderQC(self)

    def isQC(self): return True


class QMToken(UsfmToken):
    def renderOn(self, printer):
        return printer.renderQM(self)

    def isQM(self): return True


class QM1Token(UsfmToken):
    def renderOn(self, printer):
        return printer.renderQM1(self)

    def isQM1(self): return True


class QM2Token(UsfmToken):
    def renderOn(self, printer):
        return printer.renderQM2(self)

    def isQM2(self): return True


class QM3Token(UsfmToken):
    def renderOn(self, printer):
        return printer.renderQM3(self)

    def isQM3(self): return True


class QRToken(UsfmToken):
    def renderOn(self, printer):
        return printer.renderQR(self)

    def isQR(self): return True


class QSSToken(UsfmToken):
    def renderOn(self, printer):
        return printer.renderQSS(self)

    def isQSS(self): return True


class QSEToken(UsfmToken):
    def renderOn(self, printer):
        return printer.renderQSE(self)

    def isQSE(self): return True


class QTSToken(UsfmToken):
    def renderOn(self, printer):
        return printer.renderQTS(self)

    def isQTS(self): return True


class QTEToken(UsfmToken):
    def renderOn(self, printer):
        return printer.renderQTE(self)

    def isQTE(self): return True


class NBToken(UsfmToken):
    def renderOn(self, printer):
        return printer.renderNB(self)

    def isNB(self): return True


class FSToken(UsfmToken):
    def renderOn(self, printer):
        return printer.renderFS(self)

    def isFS(self): return True


class FRToken(UsfmToken):
    def renderOn(self, printer):
        return printer.renderFR(self)

    def isFR(self): return True


class FREToken(UsfmToken):
    def renderOn(self, printer):
        return printer.renderFRE(self)

    def isFRE(self): return True


class FKToken(UsfmToken):
    def renderOn(self, printer):
        return printer.renderFK(self)

    def isFK(self): return True


class FTToken(UsfmToken):
    def renderOn(self, printer):
        return printer.renderFT(self)

    def isFT(self): return True


class FQToken(UsfmToken):
    def renderOn(self, printer):
        return printer.renderFQ(self)

    def isFQ(self): return True


class FQAToken(UsfmToken):
    def renderOn(self, printer):
        return printer.renderFQA(self)

    def isFQA(self): return True


class FQAEToken(UsfmToken):
    def renderOn(self, printer):
        return printer.renderFQAE(self)

    def isFQAE(self): return True


class FQBToken(UsfmToken):
    def renderOn(self, printer):
        return printer.renderFQAE(self)

    def isFQB(self): return True


class FEToken(UsfmToken):
    def renderOn(self, printer):
        return printer.renderFE(self)

    def isFE(self): return True


class FPToken(UsfmToken):
    def renderOn(self, printer):
        return printer.renderFP(self)

    def isFP(self): return True


class ISToken(UsfmToken):
    def renderOn(self, printer):
        return printer.renderIS(self)

    def isIS(self): return True


class IEToken(UsfmToken):
    def renderOn(self, printer):
        return printer.renderIE(self)

    def isIE(self): return True


class BDSToken(UsfmToken):
    def renderOn(self, printer):
        return printer.renderBDS(self)

    def isBDS(self): return True


class BDEToken(UsfmToken):
    def renderOn(self, printer):
        return printer.renderBDE(self)

    def isBDE(self): return True


class BDITSToken(UsfmToken):
    def renderOn(self, printer):
        return printer.renderBDITS(self)

    def isBDITS(self): return True


class BDITEToken(UsfmToken):
    def renderOn(self, printer):
        return printer.renderBDITE(self)

    def isBDITE(self): return True


class LIToken(UsfmToken):
    def renderOn(self, printer):
        return printer.renderLI(self)

    def isLI(self): return True


class LI1Token(UsfmToken):
    def renderOn(self, printer):
        return printer.renderLI1(self)

    def isLI1(self): return True


class LI2Token(UsfmToken):
    def renderOn(self, printer):
        return printer.renderLI2(self)

    def isLI1(self): return True


class LI3Token(UsfmToken):
    def renderOn(self, printer):
        return printer.renderLI3(self)

    def isLI1(self): return True


class LI4Token(UsfmToken):
    def renderOn(self, printer):
        return printer.renderLI4(self)

    def isLI1(self): return True


class DToken(UsfmToken):
    def renderOn(self, printer):
        return printer.renderD(self)

    def isD(self): return True


class SPToken(UsfmToken):
    def renderOn(self, printer):
        return printer.renderSP(self)

    def isSP(self): return True


class ADDSToken(UsfmToken):
    def renderOn(self, printer):
        return printer.renderADDS(self)

    def isADDS(self): return True


class ADDEToken(UsfmToken):
    def renderOn(self, printer):
        return printer.renderADDE(self)

    def isADDE(self): return True


class NDSToken(UsfmToken):
    def renderOn(self, printer):
        return printer.renderNDS(self)

    def isNDS(self): return True


class NDEToken(UsfmToken):
    def renderOn(self, printer):
        return printer.renderNDE(self)

    def isNDE(self): return True


class PBRToken(UsfmToken):
    def renderOn(self, printer):
        return printer.renderPBR(self)

    def isPBR(self): return True


# Cross References
class XSToken(UsfmToken):
    def renderOn(self, printer):
        return printer.renderXS(self)

    def isXS(self): return True


class XDCSToken(UsfmToken):
    def renderOn(self, printer):
        return printer.renderXDCS(self)

    def isXDCS(self): return True


class XDCEToken(UsfmToken):
    def renderOn(self, printer):
        return printer.renderXDCE(self)

    def isXDCE(self): return True


class XOToken(UsfmToken):
    def renderOn(self, printer):
        return printer.renderXO(self)

    def isXO(self): return True


class XTToken(UsfmToken):
    def renderOn(self, printer):
        return printer.renderXT(self)

    def isXT(self): return True


class XEToken(UsfmToken):
    def renderOn(self, printer):
        return printer.renderXE(self)

    def isXE(self): return True


class MToken(UsfmToken):
    def renderOn(self, printer):
        return printer.renderM(self)

    def isM(self): return True


# Transliterated Words
class TLSToken(UsfmToken):
    def renderOn(self, printer):
        return printer.renderTLS(self)

    def isTLS(self): return True


class TLEToken(UsfmToken):
    def renderOn(self, printer):
        return printer.renderTLE(self)

    def isTLE(self): return True


# Indenting paragraphs
class PIToken(UsfmToken):
    def renderOn(self, printer):
        return printer.renderPI(self)

    def isPI(self): return True


class PI2Token(UsfmToken):
    def renderOn(self, printer):
        return printer.renderPI2(self)

    def isPI2(self): return True


# Small caps
class SCSToken(UsfmToken):
    def renderOn(self, printer):
        return printer.renderSCS(self)

    def isSCS(self): return True


class SCEToken(UsfmToken):
    def renderOn(self, printer):
        return printer.renderSCE(self)

    def isSCE(self): return True


# REMarks
class REMToken(UsfmToken):
    def renderOn(self, printer): return printer.renderREM(self)

    def isREM(self): return True


# Tables
class TRToken(UsfmToken):
    def renderOn(self, printer):
        return printer.renderTR(self)

    def isTR(self): return True


class TH1Token(UsfmToken):
    def renderOn(self, printer):
        return printer.renderTH1(self)

    def isTH1(self): return True


class TH2Token(UsfmToken):
    def renderOn(self, printer):
        return printer.renderTH2(self)

    def isTH2(self): return True


class TH3Token(UsfmToken):
    def renderOn(self, printer):
        return printer.renderTH3(self)

    def isTH3(self): return True


class TH4Token(UsfmToken):
    def renderOn(self, printer):
        return printer.renderTH4(self)

    def isTH4(self): return True


class TH5Token(UsfmToken):
    def renderOn(self, printer):
        return printer.renderTH5(self)

    def isTH5(self): return True


class TH6Token(UsfmToken):
    def renderOn(self, printer):
        return printer.renderTH6(self)

    def isTH6(self): return True


class THR1Token(UsfmToken):
    def renderOn(self, printer):
        return printer.renderTHR1(self)

    def isTHR1(self): return True


class THR2Token(UsfmToken):
    def renderOn(self, printer):
        return printer.renderTHR2(self)

    def isTHR2(self): return True


class THR3Token(UsfmToken):
    def renderOn(self, printer):
        return printer.renderTHR3(self)

    def isTHR3(self): return True


class THR4Token(UsfmToken):
    def renderOn(self, printer):
        return printer.renderTHR4(self)

    def isTHR4(self): return True


class THR5Token(UsfmToken):
    def renderOn(self, printer):
        return printer.renderTHR5(self)

    def isTHR5(self): return True


class THR6Token(UsfmToken):
    def renderOn(self, printer):
        return printer.renderTHR6(self)

    def isTHR6(self): return True


class TC1Token(UsfmToken):
    def renderOn(self, printer):
        return printer.renderTC1(self)

    def isTC1(self): return True


class TC2Token(UsfmToken):
    def renderOn(self, printer):
        return printer.renderTC2(self)

    def isTC2(self): return True


class TC3Token(UsfmToken):
    def renderOn(self, printer):
        return printer.renderTC3(self)

    def isTC3(self): return True


class TC4Token(UsfmToken):
    def renderOn(self, printer):
        return printer.renderTC4(self)

    def isTC4(self): return True


class TC5Token(UsfmToken):
    def renderOn(self, printer):
        return printer.renderTC5(self)

    def isTC5(self): return True


class TC6Token(UsfmToken):
    def renderOn(self, printer):
        return printer.renderTC6(self)

    def isTC6(self): return True


class TCR1Token(UsfmToken):
    def renderOn(self, printer):
        return printer.renderTCR1(self)

    def isTCR1(self): return True


class TCR2Token(UsfmToken):
    def renderOn(self, printer):
        return printer.renderTCR2(self)

    def isTCR2(self): return True


class TCR3Token(UsfmToken):
    def renderOn(self, printer):
        return printer.renderTCR3(self)

    def isTCR3(self): return True


class TCR4Token(UsfmToken):
    def renderOn(self, printer):
        return printer.renderTCR4(self)

    def isTCR4(self): return True


class TCR5Token(UsfmToken):
    def renderOn(self, printer):
        return printer.renderTCR5(self)

    def isTCR5(self): return True


class TCR6Token(UsfmToken):
    def renderOn(self, printer):
        return printer.renderTCR6(self)

    def isTCR6(self): return True


# Introductions
class IS1_Token(UsfmToken):
    def renderOn(self, printer): return printer.render_is1(self)

    def is_is1(self): return True


class IMT1_Token(UsfmToken):
    def renderOn(self, printer): return printer.render_imt1(self)

    def is_imt1(self): return True


class IMT2_Token(UsfmToken):
    def renderOn(self, printer): return printer.render_imt2(self)

    def is_imt2(self): return True


class IMT3_Token(UsfmToken):
    def renderOn(self, printer): return printer.render_imt3(self)

    def is_imt3(self): return True


class IP_Token(UsfmToken):
    def renderOn(self, printer): return printer.render_ip(self)

    def is_ip(self): return True


class IOT_Token(UsfmToken):
    def renderOn(self, printer): return printer.render_iot(self)

    def is_iot(self): return True


class IO1_Token(UsfmToken):
    def renderOn(self, printer): return printer.render_io1(self)

    def is_io1(self): return True


class IO2_Token(UsfmToken):
    def renderOn(self, printer): return printer.render_io2(self)

    def is_io2(self): return True


class IOR_S_Token(UsfmToken):
    def renderOn(self, printer): return printer.render_ior_s(self)

    def is_ior_s(self): return True


class IOR_E_Token(UsfmToken):
    def renderOn(self, printer): return printer.render_ior_e(self)

    def is_ior_e(self): return True


# Quoted book title
class BK_S_Token(UsfmToken):
    def renderOn(self, printer): return printer.render_bk_s(self)

    def is_bk_s(self): return True


class BK_E_Token(UsfmToken):
    def renderOn(self, printer): return printer.render_bk_e(self)

    def is_bk_e(self): return True



# -*- coding: utf-8 -*-
#

import abstractRenderer
import codecs
import books

#
#   Renders as set of web pages
#

class DummyFile(object):
    def close(self):
        pass
    def write(self, str):
        pass

class HTMLRenderer(abstractRenderer.AbstractRenderer):
    
    def __init__(self, inputDir, outputDir):
        # Unset
        self.f = DummyFile()  # output file stream
        # IO
        self.outputDir = outputDir
        self.inputDir = inputDir
        # Position
        self.cb = u''    # Current Book
        self.cc = u'001'    # Current Chapter
        self.cv = u'001'    # Currrent Verse
        self.indentFlag = False
        
    def render(self):
        # Write index
        self.f = open(self.outputDir + u'/index.html', 'w')
        self.write(indexPage)
        self.close()
        self.f = DummyFile()
        # Write pages
        self.loadUSFM(self.inputDir)
        self.run()
        self.f.close()
        
    def writeLog(self, s):
        print s
        
    # Support    
        
    def openFile(self, bookID):
        self.f = open(self.outputDir + u'/b' + bookID + u'.html', 'w')
        
    def close(self): 
        self.f.close()
 
    def write(self, unicodeString):
        self.f.write(unicodeString.encode('utf-8'))

    def writeIndent(self, level):
        if level == 0:
            self.indentFlag = False
            self.write(u'<br /><br />')
            return 
        if not self.indentFlag:
            self.indentFlag = True
            self.write(u'<br />')
        self.write(u'<br />')
        self.write(u'&nbsp;&nbsp;&nbsp;&nbsp;' * level)

    def renderID(self, token): 
        self.write(footer)
        self.close()
        self.cb = books.bookKeyForIdValue(token.value)
        self.openFile(self.cb)
        self.write(header)
        self.indentFlag = False
    def renderMT(self, token):      self.write(u'</p><h1>' + token.value + u'</h1><p>')
    def renderMT2(self, token):      self.write(u'</p><h2>' + token.value + u'</h2><p>')
    def renderMS(self, token):      self.write(u'</p><h4>' + token.value + u'</h4><p><br />')
    def renderMS2(self, token):     self.write(u'</p><h5>' + token.value + u'</h5><p><br />')
    def renderP(self, token):
        self.indentFlag = False
        self.write(u'<br /><br />')
    def renderS(self, token):
        self.indentFlag = False
        self.write(u'</p><p align="center">_________________________</p><p>')
    def renderS2(self, token):
        self.indentFlag = False
        self.write(u'</p><p align="center">----</p><p>')
    def renderC(self, token):
        self.cc = token.value.zfill(3)
        self.write(u'<h2 class="c-num">' + token.value + u'</h2><p>')
    def renderV(self, token):
        self.cv = token.value.zfill(3)
        if self.cv == u'001':
            self.write(u'\n<span class="verse" rel="v' + self.cb + self.cc + self.cv + u'"><span class="v-num-1">' + token.value + u'&nbsp;</span>\n')
        else:
            self.write(u'</span>\n<span class="verse" rel="v' + self.cb + self.cc + self.cv + u'"><span class="v-num">' + token.value + u'&nbsp;</span>\n')
    def renderWJS(self, token):     self.write(u'<span class="woc">')
    def renderWJE(self, token):     self.write(u'</span>')
    def renderTEXT(self, token):    self.write(u" " + token.value + u" ")
    def renderQ(self, token):       self.writeIndent(1)
    def renderQ1(self, token):      self.writeIndent(1)
    def renderQ2(self, token):      self.writeIndent(2)
    def renderQ3(self, token):      self.writeIndent(3)
    def renderNB(self, token):      self.writeIndent(0)
    def renderB(self, token):       self.write(u'<br /><br />')
    def renderIS(self, token):      self.write(u'<i>')
    def renderIE(self, token):      self.write(u'</i>')
    def renderPBR(self, token):     self.write(u'<br />')

#
#  Structure
#

indexPage = ur"""
        <!DOCTYPE html PUBLIC "-//W3C//DTD XHTML 1.0 Strict//EN" "http://www.w3.org/TR/xhtml1/DTD/xhtml1-strict.dtd">
        <html xmlns="http://www.w3.org/1999/xhtml">
        <head>
            <meta http-equiv="content-type" content="text/html; charset=utf-8" />
            <title>Bible</title>
        </head>
		<style media="all" type="text/css">
		h2.c-num { position:absolute; margin-left:-30px; color: #AAAAAA; }
		span.v-num-1 { display : none; }
		span.v-num { color: #AAAAAA; font-size: small}
		a { color:green; text-decoration:none;}
		p { font-family: Verdana, Helvetica, Arial, Sans-Serif;}
		li { font-family: Verdana, Helvetica, Arial, Sans-Serif;}
		h1 { font-family: Verdana, Helvetica, Arial, Sans-Serif; }
		h2 { font-family: Verdana, Helvetica, Arial, Sans-Serif; }
		h3 { font-family: Verdana, Helvetica, Arial, Sans-Serif; }
		h4 { font-family: Verdana, Helvetica, Arial, Sans-Serif; }
		</style>
        <body>
        <div style="position:absolute; margin-left:5; width:190px; background-color:#E6FAE6;">
		<h2>Bible</h2>

        <p><b>Old Testament</b></p>
        <ul>
        <li><a href="b001.html">Genesis</li>
        <li><a href="b002.html">Exodus</li>
        <li><a href="b003.html">Leviticus</li>
        <li><a href="b004.html">Numbers</li>
        <li><a href="b005.html">Deuteronomy</li>
        <li><a href="b006.html">Joshua</li>
        <li><a href="b007.html">Judges</li>
        <li><a href="b008.html">Ruth</a></li>
        <li><a href="b009.html">1 Samuel</li>
        <li><a href="b010.html">2 Samuel</li>
        <li><a href="b011.html">1 Kings</li>
        <li><a href="b012.html">2 Kings</li>
        <li><a href="b013.html">1 Chronicles</li>
        <li><a href="b014.html">2 Chronicles</li>
        <li><a href="b015.html">Ezra</li>
        <li><a href="b016.html">Nehemiah</li>
        <li><a href="b017.html">Esther</a></li>
        <li><a href="b018.html">Job</li>
        <li><a href="b019.html">Psalms</a></li>
        <li><a href="b020.html">Proverbs</li>
        <li><a href="b021.html">Ecclesiastes</li>
        <li><a href="b022.html">Song of Solomon</li>
        <li><a href="b023.html">Isaiah</a></li>
        <li><a href="b024.html">Jeremiah</li>
        <li><a href="b025.html">Lamentations</li>
        <li><a href="b026.html">Ezekiel</li>
        <li><a href="b027.html">Daniel</a></li>
        <li><a href="b028.html">Hosea</li>
        <li><a href="b029.html">Joel</a></li>
        <li><a href="b030.html">Amos</li>
        <li><a href="b031.html">Obadiah</a></li>
        <li><a href="b032.html">Jonah</a></li>
        <li><a href="b033.html">Micah</li>
        <li><a href="b034.html">Nahum</a></li>
        <li><a href="b035.html">Habakkuk</a></li>
        <li><a href="b036.html">Zephaniah</a></li>
        <li><a href="b037.html">Haggai</a></li>
        <li><a href="b038.html">Zechariah</li>
        <li><a href="b039.html">Malachi</li>
        </ul>            

        <p><b>New Testament</b></p>
        <ul>
        <li><a href="b040.html">Matthew</a></li>
        <li><a href="b041.html">Mark</a></li>
        <li><a href="b042.html">Luke</a></li>
        <li><a href="b043.html">John</a></li>
        <li><a href="b044.html">Acts</a></li>
        <li><a href="b045.html">Romans</a></li>
        <li><a href="b046.html">1 Corinthians</a></li>
        <li><a href="b047.html">2 Corinthians</a></li>
        <li><a href="b048.html">Galatians</a></li>
        <li><a href="b049.html">Ephesians</a></li>
        <li><a href="b050.html">Philippians</a></li>
        <li><a href="b051.html">Colossians</a></li>
        <li><a href="b052.html">1 Thessalonians</a></li>
        <li><a href="b053.html">2 Thessalonians</a></li>
        <li><a href="b054.html">1 Timothy</a></li>
        <li><a href="b055.html">2 Timothy</a></li>
        <li><a href="b056.html">Titus</a></li>
        <li><a href="b057.html">Philemon</a></li>
        <li><a href="b058.html">Hebrews</a></li>
        <li><a href="b059.html">James</a></li>
        <li><a href="b060.html">1 Peter</a></li>
        <li><a href="b061.html">2 Peter</a></li>
        <li><a href="b062.html">1 John</a></li>
        <li><a href="b063.html">2 John</a></li>
        <li><a href="b064.html">3 John</a></li>
        <li><a href="b065.html">Jude</a></li>
        <li><a href="b066.html">Revelation</a></li>
        </ul>
        </div>
        <div style="margin-left:230px; margin-right:80px">
        </div>
        </body>
        </html>
"""

header = ur"""
        <!DOCTYPE html PUBLIC "-//W3C//DTD XHTML 1.0 Strict//EN" "http://www.w3.org/TR/xhtml1/DTD/xhtml1-strict.dtd">
        <html xmlns="http://www.w3.org/1999/xhtml">
        <head>
            <meta http-equiv="content-type" content="text/html; charset=utf-8" />
            <title>Bible</title>
        </head>
		<style media="all" type="text/css">
		h2.c-num { position:absolute; margin-left:-30px; color: #AAAAAA; }
		span.v-num-1 { display : none; }
		span.v-num { color: #AAAAAA; font-size: small}
		a { color:green; text-decoration:none;}
		p { font-family: Verdana, Helvetica, Arial, Sans-Serif;}
		li { font-family: Verdana, Helvetica, Arial, Sans-Serif;}
		h1 { font-family: Verdana, Helvetica, Arial, Sans-Serif; }
		h2 { font-family: Verdana, Helvetica, Arial, Sans-Serif; }
		h3 { font-family: Verdana, Helvetica, Arial, Sans-Serif; }
		h4 { font-family: Verdana, Helvetica, Arial, Sans-Serif; }
		</style>
        <body>
        <div style="position:absolute; margin-left:5; width:190px; background-color:#E6FAE6;">
		<h2>Bible</h2>

        <p><b>Old Testament</b></p>
        <ul>
        <li><a href="b001.html">Genesis</li>
        <li><a href="b002.html">Exodus</li>
        <li><a href="b003.html">Leviticus</li>
        <li><a href="b004.html">Numbers</li>
        <li><a href="b005.html">Deuteronomy</li>
        <li><a href="b006.html">Joshua</li>
        <li><a href="b007.html">Judges</li>
        <li><a href="b008.html">Ruth</a></li>
        <li><a href="b009.html">1 Samuel</li>
        <li><a href="b010.html">2 Samuel</li>
        <li><a href="b011.html">1 Kings</li>
        <li><a href="b012.html">2 Kings</li>
        <li><a href="b013.html">1 Chronicles</li>
        <li><a href="b014.html">2 Chronicles</li>
        <li><a href="b015.html">Ezra</li>
        <li><a href="b016.html">Nehemiah</li>
        <li><a href="b017.html">Esther</a></li>
        <li><a href="b018.html">Job</li>
        <li><a href="b019.html">Psalms</a></li>
        <li><a href="b020.html">Proverbs</li>
        <li><a href="b021.html">Ecclesiastes</li>
        <li><a href="b022.html">Song of Solomon</li>
        <li><a href="b023.html">Isaiah</a></li>
        <li><a href="b024.html">Jeremiah</li>
        <li><a href="b025.html">Lamentations</li>
        <li><a href="b026.html">Ezekiel</li>
        <li><a href="b027.html">Daniel</a></li>
        <li><a href="b028.html">Hosea</li>
        <li><a href="b029.html">Joel</a></li>
        <li><a href="b030.html">Amos</li>
        <li><a href="b031.html">Obadiah</a></li>
        <li><a href="b032.html">Jonah</a></li>
        <li><a href="b033.html">Micah</li>
        <li><a href="b034.html">Nahum</a></li>
        <li><a href="b035.html">Habakkuk</a></li>
        <li><a href="b036.html">Zephaniah</a></li>
        <li><a href="b037.html">Haggai</a></li>
        <li><a href="b038.html">Zechariah</li>
        <li><a href="b039.html">Malachi</li>
        </ul>            

        <p><b>New Testament</b></p>
        <ul>
        <li><a href="b040.html">Matthew</a></li>
        <li><a href="b041.html">Mark</a></li>
        <li><a href="b042.html">Luke</a></li>
        <li><a href="b043.html">John</a></li>
        <li><a href="b044.html">Acts</a></li>
        <li><a href="b045.html">Romans</a></li>
        <li><a href="b046.html">1 Corinthians</a></li>
        <li><a href="b047.html">2 Corinthians</a></li>
        <li><a href="b048.html">Galatians</a></li>
        <li><a href="b049.html">Ephesians</a></li>
        <li><a href="b050.html">Philippians</a></li>
        <li><a href="b051.html">Colossians</a></li>
        <li><a href="b052.html">1 Thessalonians</a></li>
        <li><a href="b053.html">2 Thessalonians</a></li>
        <li><a href="b054.html">1 Timothy</a></li>
        <li><a href="b055.html">2 Timothy</a></li>
        <li><a href="b056.html">Titus</a></li>
        <li><a href="b057.html">Philemon</a></li>
        <li><a href="b058.html">Hebrews</a></li>
        <li><a href="b059.html">James</a></li>
        <li><a href="b060.html">1 Peter</a></li>
        <li><a href="b061.html">2 Peter</a></li>
        <li><a href="b062.html">1 John</a></li>
        <li><a href="b063.html">2 John</a></li>
        <li><a href="b064.html">3 John</a></li>
        <li><a href="b065.html">Jude</a></li>
        <li><a href="b066.html">Revelation</a></li>
        </ul>
        </div>
        <div style="margin-left:230px; margin-right:80px">
        """

footer = ur"""
        </p></div></body>   
        """


# -*- coding: utf-8 -*-
#

from __future__ import print_function, unicode_literals
from .abstractRenderer import AbstractRenderer
import codecs
import logging

#
#   Simplest renderer. Ignores everything except ascii text.
#


# noinspection PyPep8Naming,PyUnusedLocal
class USXRenderer(AbstractRenderer):
    def __init__(self, input_dir, output_path, output_name, by_book_flag):
        # Unset
        self.f = None  # output file stream
        # IO
        self.outputFilePath = output_path
        self.outputFileName = output_name
        self.outputFileExt = '.usx'
        self.inputDir = input_dir
        self.byBook = by_book_flag
        # Position
        self.currentC = 1
        self.currentV = 1
        self.book = ''        # book id
        self.renderBook = ''  # book name
        # Flags
        self.printerState = {'c': False, 'p': False, 'pi': False, 'pi2': False, 'q': False, 'li': False, 'row': False,
                             'cell': False, 'table': False}
        # Errors
        self.error = ''
        self.__logger = logging.getLogger('usfm_tools')

    def render(self):
        self.loadUSFM(self.inputDir)
        if self.byBook:
            self.__logger.info('Creating One File Per Book\n')
            for bookName in self.booksUsfm:
                self.f = codecs.open(self.outputFilePath + bookName + self.outputFileExt, 'w', 'utf_8_sig')
                self.renderBook = bookName
                self.run()
                self.f.write(self.stop_all())
                self.f.close()
        else:
            self.__logger.info('Concatenating Books into ' + self.outputFileName + self.outputFileExt + '\n')
            self.f = codecs.open(self.outputFilePath + self.outputFileName + self.outputFileExt, 'w', 'utf_8_sig')
            self.run()
            self.f.write(self.stop_p() + self.stop_pi() + self.stop_pi2() + self.stop_q() + self.stop_li())
            self.f.close()
        self.__logger.info('')

    def writeLog(self, s):
        self.__logger.info(s)

    # Support

    def indent(self):
        if self.printerState['p']:
            return '  '
        else:
            return ''

    def start_c(self):
        if not self.printerState['c']:
            self.printerState['c'] = True
        return '\n<chapter number="' + str(self.currentC) + '"'

    def stop_c(self):
        if self.printerState['c']:
            self.printerState['c'] = False
            return ' style="c" />'
        else:
            return ''

    def start_p(self):
        if not self.printerState['p']:
            self.printerState['p'] = True
        return '\n<para style="p">'

    def stop_p(self):
        if self.printerState['p']:
            self.printerState['p'] = False
            return '</para>'
        else:
            return ''

    def start_pi(self):
        if not self.printerState['pi']:
            self.printerState['pi'] = True
        return '\n<para style="pi">'

    def stop_pi(self):
        if self.printerState['pi']:
            self.printerState['pi'] = False
            return '</para>'
        else:
            return ''

    def start_pi2(self):
        if not self.printerState['pi2']:
            self.printerState['pi2'] = True
        return '\n<para style="pi2">'

    def stop_pi2(self):
        if self.printerState['pi2']:
            self.printerState['pi2'] = False
            return '</para>'
        else:
            return ''

    def start_q(self, n):
        if not self.printerState['q']:
            self.printerState['q'] = True
        return '\n<para style="q' + str(n) + '">'

    def start_qm(self, n):
        if not self.printerState['q']:
            self.printerState['q'] = True
        return '\n<para style="qm' + str(n) + '">'

    def stop_q(self):
        if self.printerState['q']:
            self.printerState['q'] = False
            return '</para>'
        else:
            return ''

    def start_li(self, n):
        if not self.printerState['li']:
            self.printerState['li'] = True
        return '\n<para style="li' + str(n) + '">'

    def stop_li(self):
        if self.printerState['li']:
            self.printerState['li'] = False
            return '</para>'
        else:
            return ''

    def start_cell(self, style, align):
        if not self.printerState['cell']:
            self.printerState['cell'] = True
            return '\n    <cell style="' + style + '" align="' + align + '">'
        else:
            return ''

    def stop_cell(self):
        if self.printerState['cell']:
            self.printerState['cell'] = False
            return '</cell>'
        else:
            return ''

    def start_row(self):
        if not self.printerState['row']:
            self.printerState['row'] = True
        return '\n  <row style="tr">'

    def stop_row(self):
        if self.printerState['row']:
            self.printerState['row'] = False
            return '\n  </row>'
        else:
            return ''

    def start_table(self):
        if not self.printerState['table']:
            self.printerState['table'] = True
            return '\n<table>'
        else:
            return ''

    def stop_table(self):
        if self.printerState['table']:
            self.printerState['table'] = False
            return '\n</table>'
        else:
            return ''

    def stop_all(self):
        return self.stop_c() + self.stop_p() + self.stop_pi() + self.stop_pi2() + self.stop_q() + self.stop_li() + \
               self.stop_cell() + self.stop_row() + self.stop_table()

    def stop_all_to_cell(self):
        return self.stop_c() + self.stop_p() + self.stop_pi() + self.stop_pi2() + self.stop_q() + self.stop_li() + \
               self.stop_cell()

    # noinspection PyMethodMayBeStatic
    def escape(self, s):
        return s

    def renderTEXT(self, token):
        self.f.write(self.escape(token.value))

    def renderH(self, token):
        self.book = token.getValue()

    def renderMT(self, token):
        self.f.write(self.stop_all() + '\n<para style="mt">' + token.value.upper() + '</para>')

    def renderMT1(self, token):
        self.f.write(self.stop_all() + '\n<para style="mt1">' + token.value.upper() + '</para>')

    def renderMT2(self, token):
        self.f.write(self.stop_all() + '\n<para style="mt2">' + token.value.upper() + '</para>')

    def renderMT3(self, token):
        self.f.write(self.stop_all() + '\n<para style="mt3">' + token.value.upper() + '</para>')

    def renderMT4(self, token):
        self.f.write(self.stop_all() + '\n<para style="mt4">' + token.value.upper() + '</para>')

    def renderMT5(self, token):
        self.f.write(self.stop_all() + '\n<para style="mt5">' + token.value.upper() + '</para>')

    def renderMS(self, token):
        self.f.write(self.stop_all() + '\n<para style="ms">' + token.value + '</para>')

    def renderMS1(self, token):
        self.f.write(self.stop_all() + '\n<para style="ms1">' + token.value + '</para>')

    def renderMS2(self, token):
        self.f.write(self.stop_all() + '\n<para style="ms2">' + token.value + '</para>')

    def renderMS3(self, token):
        self.f.write(self.stop_all() + '\n<para style="ms3">' + token.value + '</para>')

    def renderMS4(self, token):
        self.f.write(self.stop_all() + '\n<para style="ms4">' + token.value + '</para>')

    def renderMS5(self, token):
        self.f.write(self.stop_all() + '\n<para style="ms5">' + token.value + '</para>')

    def renderP(self, token):
        self.f.write(self.stop_all() + self.start_p())

    def renderPI(self, token):
        self.f.write(self.stop_all() + self.start_pi())

    def renderPI2(self, token):
        self.f.write(self.stop_all() + self.start_pi2())

    def renderB(self, token):
        self.f.write(self.stop_all() + '\n<para style="b" />')

    def renderS(self, token):
        self.f.write(self.stop_all() + '\n\n<para style="s">' + token.value + '</para>')

    def renderS1(self, token):
        self.f.write(self.stop_all() + '\n\n<para style="s1">' + token.value + '</para>')

    def renderS2(self, token):
        self.f.write(self.stop_all() + '\n\n<para style="s2">' + token.value + '</para>')

    def renderS3(self, token):
        self.f.write(self.stop_all() + '\n\n<para style="s3">' + token.value + '</para>')

    def renderS4(self, token):
        self.f.write(self.stop_all() + '\n\n<para style="s4">' + token.value + '</para>')

    def renderS5(self, token):
        self.f.write(self.stop_c() + '\n<note caller="u" style="s5"></note>')

    def renderC(self, token):
        self.currentC = token.value
        self.currentV = '0'
        self.f.write(self.stop_all() + self.start_c())

    def renderCAS(self, token):
        self.f.write(' altnumber="')

    # noinspection PyUnusedLocal
    def renderCAE(self, token):
        self.f.write('"')

    def renderCL(self, token):
        self.f.write(self.stop_all() + '\n\n<para style="cl">' + token.value + '</para>')

    def renderV(self, token):
        self.currentV = token.value
        self.f.write(
            self.stop_c() + '\n' + self.indent() + '<verse number="' + token.value + '" style="v" />')

    def renderQ(self, token):
        self.renderQ1(token)

    def renderQ1(self, token):
        self.f.write(self.stop_all() + self.start_q(1))

    def renderQ2(self, token):
        self.f.write(self.stop_all() + self.start_q(2))

    def renderQ3(self, token):
        self.f.write(self.stop_all() + self.start_q(3))

    def renderQ4(self, token):
        self.f.write(self.stop_all() + self.start_q(4))

    def renderQA(self, token):
        self.f.write(self.stop_all() + '\n<para style="qa">' + token.value + '</para>')

    def renderQAC(self, token):
        self.f.write(self.stop_c() + '\n<char style="qac">' + self.escape(token.value) + '</char>')

    def renderQC(self, token):
        self.f.write(self.stop_all() + '\n<para style="qc">' + token.value + '</para>')

    def renderQM(self, token):
        self.renderQM1(token)

    def renderQM1(self, token):
        self.f.write(self.stop_all() + self.start_qm(1))

    def renderQM2(self, token):
        self.f.write(self.stop_all() + self.start_qm(2))

    def renderQM3(self, token):
        self.f.write(self.stop_all() + self.start_qm(3))

    def renderQR(self, token):
        self.f.write(self.stop_all() + '\n<para style="qr">' + token.value + '</para>')

    def renderQSS(self, token):
        self.f.write(self.stop_c() + '\n<char style="qs">')

    def renderQSE(self, token):
        self.f.write(self.stop_c() + '</char>')

    def renderQTS(self, token):
        self.f.write(self.stop_c() + '<char style="qt">')

    def renderQTE(self, token):
        self.f.write(self.stop_c() + '</char>')

    def renderNB(self, token):
        self.f.write(self.stop_c() + '\n<char style="nb">' + self.indent() + '</char>')

    def renderLI(self, token):
        self.renderLI1(token)

    def renderLI1(self, token):
        self.f.write(self.stop_all() + self.start_li(1))

    def renderLI2(self, token):
        self.f.write(self.stop_all() + self.start_li(2))

    def renderLI3(self, token):
        self.f.write(self.stop_all() + self.start_li(3))

    def renderLI4(self, token):
        self.f.write(self.stop_all() + self.start_li(4))

    def renderPBR(self, token):
        self.f.write(self.stop_c() + '\n')

    def renderBDS(self, token):
        self.f.write(self.stop_c() + '<char style="bd">')

    def renderBDE(self, token):
        self.f.write(self.stop_c() + '</char>')

    def renderBDITS(self, token):
        # noinspection SpellCheckingInspection
        self.f.write(self.stop_c() + '<char style="bdit">')

    def renderBDITE(self, token):
        self.f.write(self.stop_c() + '</char>')

    def renderEMS(self, token):
        self.f.write(self.stop_c() + '<char style="em">')

    def renderEME(self, token):
        self.f.write(self.stop_c() + '</char>')

    def renderITS(self, token):
        self.f.write(self.stop_c() + '<char style="it">')

    def renderITE(self, token):
        self.f.write(self.stop_c() + '</char>')

    def renderNOS(self, token):
        self.f.write(self.stop_c() + '<char style="no">')

    def renderNOE(self, token):
        self.f.write(self.stop_c() + '</char>')

    def renderSCS(self, token):
        self.f.write(self.stop_c() + '<char style="sc">')

    def renderSCE(self, token):
        self.f.write(self.stop_c() + '</char>')

    def renderFS(self, token):
        self.f.write(self.stop_c() + '\n<note caller="+" style="f">')

    def renderFE(self, token):
        self.f.write(self.stop_c() + '\n</note>')

    def renderFR(self, token):
        self.f.write(self.stop_c() + '\n  <char style="fr">' + self.escape(token.value) + '</char>')

    def renderFT(self, token):
        self.f.write(self.stop_c() + '\n  <char style="ft">' + self.escape(token.value) + '</char>')

    def renderFQ(self, token):
        self.f.write(self.stop_c() + '\n  <char style="fq">' + self.escape(token.value) + '</char>')

    def renderFQA(self, token):
        self.f.write(self.stop_c() + '\n  <char style="fqa">' + self.escape(token.value) + '</char>')

    def renderFP(self, token):
        self.f.write(self.stop_c() + '\n')

    def renderFQAE(self, token):
        pass

    def renderFQB(self, token):
        pass

    def renderXS(self, token):
        self.f.write(self.stop_c() + '\n<note caller="-" style="x">')

    def renderXE(self, token):
        self.f.write(self.stop_c() + '\n</note>')

    def renderXO(self, token):
        self.f.write(self.stop_c() + '\n  <char style="xo">' + self.escape(token.value) + '</char>')

    def renderXT(self, token):
        self.f.write(self.stop_c() + '\n  <char style="xt">' + self.escape(token.value) + '</char>')

    def renderTR(self, token):
        self.f.write(self.stop_all_to_cell() + self.stop_row() + self.start_table() + self.start_row())

    def renderTH1(self, token):
        self.f.write(self.stop_all_to_cell() + self.start_cell('th1', 'start'))

    def renderTH2(self, token):
        self.f.write(self.stop_all_to_cell() + self.start_cell('th2', 'start'))

    def renderTH3(self, token):
        self.f.write(self.stop_all_to_cell() + self.start_cell('th3', 'start'))

    def renderTH4(self, token):
        self.f.write(self.stop_all_to_cell() + self.start_cell('th4', 'start'))

    def renderTH5(self, token):
        self.f.write(self.stop_all_to_cell() + self.start_cell('th5', 'start'))

    def renderTH6(self, token):
        self.f.write(self.stop_all_to_cell() + self.start_cell('th6', 'start'))

    def renderTHR1(self, token):
        self.f.write(self.stop_all_to_cell() + self.start_cell('thr1', 'end'))

    def renderTHR2(self, token):
        self.f.write(self.stop_all_to_cell() + self.start_cell('thr2', 'end'))

    def renderTHR3(self, token):
        self.f.write(self.stop_all_to_cell() + self.start_cell('thr3', 'end'))

    def renderTHR4(self, token):
        self.f.write(self.stop_all_to_cell() + self.start_cell('thr4', 'end'))

    def renderTHR5(self, token):
        self.f.write(self.stop_all_to_cell() + self.start_cell('thr5', 'end'))

    def renderTHR6(self, token):
        self.f.write(self.stop_all_to_cell() + self.start_cell('thr6', 'end'))

    def renderTC1(self, token):
        self.f.write(self.stop_all_to_cell() + self.start_cell('tc1', 'start'))

    def renderTC2(self, token):
        self.f.write(self.stop_all_to_cell() + self.start_cell('tc2', 'start'))

    def renderTC3(self, token):
        self.f.write(self.stop_all_to_cell() + self.start_cell('tc3', 'start'))

    def renderTC4(self, token):
        self.f.write(self.stop_all_to_cell() + self.start_cell('tc4', 'start'))

    def renderTC5(self, token):
        self.f.write(self.stop_all_to_cell() + self.start_cell('tc5', 'start'))

    def renderTC6(self, token):
        self.f.write(self.stop_all_to_cell() + self.start_cell('tc6', 'start'))

    def renderTCR1(self, token):
        self.f.write(self.stop_all_to_cell() + self.start_cell('tcr1', 'end'))

    def renderTCR2(self, token):
        self.f.write(self.stop_all_to_cell() + self.start_cell('tcr2', 'end'))

    def renderTCR3(self, token):
        self.f.write(self.stop_all_to_cell() + self.start_cell('tcr3', 'end'))

    def renderTCR4(self, token):
        self.f.write(self.stop_all_to_cell() + self.start_cell('tcr4', 'end'))

    def renderTCR5(self, token):
        self.f.write(self.stop_all_to_cell() + self.start_cell('tcr5', 'end'))

    def renderTCR6(self, token):
        self.f.write(self.stop_all_to_cell() + self.start_cell('tcr6', 'end'))

    def render_imt1(self, token):
        self.f.write(self.stop_all() + '\n<para style="imt1">' + token.value.upper() + '</para>')

    def render_imt2(self, token):
        self.f.write(self.stop_all() + '\n<para style="imt2">' + token.value.upper() + '</para>')

    def render_imt3(self, token):
        self.f.write(self.stop_all() + '\n<para style="imt3">' + token.value.upper() + '</para>')

    def renderD(self, token):
        self.f.write(self.stop_all() + '\n<para style="d">' + token.value + '</para>')

    def renderUnknown(self, token):
        if token.value == 'v':
            self.currentV = int(self.currentV) + 1
        self.__logger.error(self.renderBook + ' ' + str(self.currentC) + ':' + str(self.currentV) +
              ' - Unknown Token: \\' + self.escape(token.value))



# -*- coding: utf-8 -*-
#

from .abstractRenderer import AbstractRenderer
import codecs
from .books import bookKeyForIdValue

#
#   UTF-8 CVS file
#

class CSVRenderer(AbstractRenderer):
    
    def __init__(self, inputDir, outputFilename):
        # Unset
        self.f = None  # output file stream
        # IO
        self.outputFilename = outputFilename
        self.inputDir = inputDir
        # Flags
        self.cb = u''    # Current Book
        self.cc = u'001'    # Current Chapter
        self.cv = u'001'    # Currrent Verse  
        self.infootnote = False
        
    def render(self):
        self.f = codecs.open(self.outputFilename, 'w', 'utf_8_sig')
        self.loadUSFM(self.inputDir)
        self.run()
        self.f.close()
        
    def writeLog(self, s):
        print(s)
                
    #   SUPPORT

    def escape(self, s):
        return '' if self.infootnote else s
            
    #   TOKENS

    def renderID(self, token): 
        self.cb = bookKeyForIdValue(token.value)
    def renderC(self, token):
        self.cc = token.value.zfill(3)
    def renderV(self, token):
        self.cv = token.value.zfill(3)
        self.f.write(u'\n' + str(int(self.cb)) + ',' + str(int(self.cc)) + ',' + self.cv   + ',')
    def renderTEXT(self, token):    self.f.write(self.escape(token.value) + ' ')
    def renderFS(self, token):      self.infootnote = True
    def renderFE(self, token):      self.infootnote = False



# -*- coding: utf-8 -*-
#

from .abstractRenderer import AbstractRenderer
import codecs
import datetime
from .books import bookKeyForIdValue

#
#   Renders using Lout (TeX alternative)
#

class LoutRenderer(AbstractRenderer):

    def __init__(self, inputDir, outputFilename):
        # Unset
        self.f = None  # output file stream
        # IO
        self.outputFilename = outputFilename
        self.inputDir = inputDir
        # Position
        self.cb = u''    # Current Book
        self.cc = u'001'    # Current Chapter
        self.cv = u'001'    # Currrent Verse
        self.indentFlag = False
        self.bookName = u''
        self.inChapter = False
        self.inSections = False
        self.inSection = False
        self.startTextType = 'drop' # Or nomal or smallcaps
        self.inDropCap = False
        self.inPoetry = False
        self.registerForNextText = u''
        self.inD = False
        self.afterLord = False

    def render(self):
        self.f = codecs.open(self.outputFilename, 'w', 'utf_8_sig')
        self.f.write(r"""@Include { oebbook }
@Book
    @Title {}
    @Author {}
    @Edition {}
    @Publisher {}
    @BeforeTitlePage {}
    @OnTitlePage {}
    @AfterTitlePage {}
    @AtEnd {}
    @InitialLanguage { English }
    @PageOrientation { Portrait }
    @PageHeaders { Titles }
    @ColumnNumber { 1 }
    @FirstPageNumber { 1 }
    @IntroFirstPageNumber { 1 }
    @OptimizePages { No }
    @GlossaryText { @Null }
    @IndexText { @Null }
    @IndexAText { @Null }
    @IndexBText { @Null }
//

""".encode('utf-8'))
        self.loadUSFM(self.inputDir)
        self.run()
        self.f.close()

    def writeLog(self, s):
        print(s)

    def write(self, unicodeString):
        self.f.write(unicodeString + '\n')

    #   SUPPORT
    #
    #
    def escape(self, s, upper=False):
        if upper:
            t = s.upper()
        else:
            t = s
        t = t.replace(u'“', u'{@Char quotedblleft}').replace(u'”', u'{@Char quotedblright}').replace(u'—', u'{@Char emdash}').replace(u'‘', u'{@Char quoteleft}').replace(u'’', u'{@Char quoteright}').replace(u'"', u'{@Char quotedbl}')
        #t = t.replace(u'"', u'{@Char quotedbl}')
        #t = t.replace(u'’', u'{@Char quoteright}')
        return t

    def close(self):
        self.closeChapter()

    def closeChapter(self):
        self.closeSections()
        if self.inChapter:          # We need to close previous chapter (ie Book)
            self.inChapter = False
            self.write(u'\n@End @Chapter\n')

    def closeSections(self):
        self.closeSection()
        if self.inSections:          # We need to close previous section
            self.inSections = False
            self.write(u'\n@EndSections\n')

    def closeSection(self):
        self.closeDropCap()
        self.closePoetry()
        if self.inSection:          # We need to close previous section
            self.inSection = False
            self.write(u'\n@End @Section\n')

    def closeDropCap(self):
        if self.inDropCap:          # We need to close previous section
            self.inDropCap = False
            self.write(u'}')

    def closeD(self):
        if self.inD:          # We need to close the D
            self.inD = False
            self.write(u'}}')

    def closePoetry(self):
        if self.inPoetry:
            self.inPoetry = False

    def writeIndent(self, level):
        self.closeD()
        self.closeDropCap()
        if not self.inPoetry:
            self.write(u'\n@DP ')
            self.inPoetry = True
        else:
            self.write(u'\n@LLP ')
        self.write(u'~ ~ ~ ~' * level)

    def formatText(self, text):
        t = text
        if len(t) < 60: self.startTextType = 'normal'  # Don't do funky things with short first lines.

        # Handle poetry lines that want to wrap
        if self.inPoetry and len(t) > 60:
            n = t.find(u' ', 50) # Give us buffer
            t = t[:n] + u'\n@LLP ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ' + t[n:]

        if self.cb == u'019': self.startTextType = 'normal'  # Don't do funky things with Psalms.
        if self.startTextType == 'drop':
            self.inDropCap = True
            self.write(t[0] + u' @DropCapTwo {')
            self.write(self.escape(self.smallCapText(t[1:])))
        elif self.startTextType == 'smallcaps':
            self.write(self.escape(self.smallCapText(t)))
        else:
            if self.afterLord and not (t[0] == "'" or t[0] == ',' or t[0] == '?'):
                self.afterLord = False
                t = u' ' + t
            self.write(self.escape(self.addNextText(t)))
        self.write(u' ')
        self.startTextType = 'normal'

    def addNextText(self, text):
        t = text
        if not self.registerForNextText == u'':
            try:
                i = t.index(u' ')
                t = t[:i] + self.registerForNextText + u' ' + t[i+1:]
            except Exception:
                t = self.registerForNextText + t
            self.registerForNextText = u''
        return t

    def smallCapText(self, s):
         i = 0
         while i < len(s):
             if i < 60:  #we are early, look for comma
                 if s[i] == u',' or s[i] == u';' or s[i] == u'.' or s[i] == u'—':
                     return u'{@S {' + self.addNextText(s[:i+1]) + u'}} ' + s[i+1:]
                 i = i + 1
             else: # look for space
                 if s[i] == ' ':
                     return u'{@S {' + self.addNextText(s[:i]) + u'}}' + s[i:]
                 i = i + 1
         return self.addNextText(s)

    def newPara(self, indent = True, outdent=False):
        # Don't indent at start of books
        if self.startTextType == 'drop': indent = False
        if self.startTextType == 'smallcaps': indent = False

        self.closeDropCap()
        if self.inPoetry:
            self.closePoetry()
            self.write(u'\n\n@DP ~\n')
        else:
            if outdent:
                self.write(u'\n\noutdent @Break { @PP }\n')
            elif indent:
                self.write(u'\n\n@PP\n')
            else:
                self.write(u'\n\n@LP\n')

    #   TOKENS
    #
    #
    def renderID(self, token):
        self.cb = bookKeyForIdValue(token.value)
        self.indentFlag = False
        self.closeChapter()
    def renderIDE(self, token):
        pass
    def renderSTS(self, token):
        pass
    def renderH(self, token):
        self.close()
        self.bookname = token.value
        self.write(u'\n@Chapter @Title { ' + self.escape(token.value) + u' } @RunningTitle { ' + self.bookname + u' } @Begin')
        self.inChapter = True
    def renderMT(self, token):
        self.write(u'\n@Display  { 21p } @Font { ' + self.escape(token.value, upper=True) + u'}')
        self.startTextType = 'drop'
    def renderMT2(self, token):
        self.write(u'\n@Display  { 13p } @Font { ' + self.escape(token.value, upper=True) + u'}')
    def renderMT3(self, token):
        self.write(u'\n@Display  { 13p } @Font { ' + self.escape(token.value, upper=True) + u'}')
    def renderMS(self, token):
        self.closeSection()
        if not self.inSections: self.write(u'@BeginSections '); self.inSections = True
        self.write(u'\n@Section @Title {' + self.escape(token.value) + u'} @Begin @LP\n')
        self.inSection = True
        if self.startTextType == 'normal': self.startTextType = 'smallcaps'
    def renderMS2(self, token):
        self.write(u'\n@Display @Heading {' + self.escape(token.value) + u'}\n')
    def renderP(self, token):
        self.newPara()
    def renderPI(self, token):
        self.newPara(outdent = True)
    def renderS(self, token):
        self.closePoetry();
        self.closeDropCap();
        self.write(u'\n@DP @CNP @Display @Heading {' + self.escape(token.value) + u'}\n')
    def renderS2(self, token):
        self.closeDropCap();
        self.write(u'\n\n@DP\n')
    def renderC(self, token):
        self.cc = token.value.zfill(3)
        self.registerForNextText = u' {@OuterNote { 10.5p @Font {@B ' + token.value + u'}}}'
    def renderV(self, token):
        self.cv = token.value.zfill(3)
        if not self.cv == u'001':   self.registerForNextText = u' {@OuterNote { 8p @Font {' + token.value + u'}}}'
    def renderTEXT(self, token):    self.formatText(token.value)
    def renderQ(self, token):       self.writeIndent(1)
    def renderQ1(self, token):      self.writeIndent(1)
    def renderQ2(self, token):      self.writeIndent(2)
    def renderQ3(self, token):      self.writeIndent(3)
    def renderNB(self, token):      self.newPara(indent = False)
    def renderB(self, token):       self.newPara(indent = False); self.inPoetry = True
    def renderFS(self, token):      self.write(u'@FootNote { ')
    def renderFE(self, token):      self.write(u' }')
    def renderFP(self, token):      self.newPara(indent = False)
    def renderIS(self, token):      self.write(u'{@I {')
    def renderIE(self, token):      self.write(u'}}')
    def renderNDS(self, token):     self.write(u'{@S {')
    def renderNDE(self, token):     self.write(u'}}'); self.afterLord = True
    def renderPBR(self, token):     self.write(u'@LLP ')
    def renderSCS(self, token):     self.write(u'{@B {')
    def renderSCE(self, token):     self.write(u'}}')
    def renderD(self, token):       self.write(u'{@I {'); self.inD = True





# -*- coding: utf-8 -*-
#

from .abstractRenderer import AbstractRenderer
import codecs

#
#   Simplest renderer. Ignores everything except ascii text.
#

class ASCIIRenderer(AbstractRenderer):
    
    def __init__(self, inputDir, outputFilename):
        # Unset
        self.f = None  # output file stream
        # IO
        self.outputFilename = outputFilename
        self.inputDir = inputDir
        # Flags
        self.d = False
        self.narrower = False
        self.inFootnote = False
        self.inX = False
        
    def render(self):
        self.f = codecs.open(self.outputFilename, 'w', 'ascii')
        self.loadUSFM(self.inputDir)
        self.run()
        self.f.close()
        
    def writeLog(self, s):
        print(s)
        
    # Support
    
    def startNarrower(self, n):
        s = u'\n'
        if not self.narrower: s = s + u'\n'
        self.narrower = True
        return s + u'    ' * n

    def stopNarrower(self):
        self.narrower = False
        return u''

    def startD(self):
        self.d = True
        return u''

    def stopD(self):
        self.d = False
        return u''
        
    def escape(self, text):
        if self.inX or self.inFootnote:
            return u''
        t = text.replace(u'‘', u"'")
        t = t.replace(u'’', u"'")
        t = t.replace(u'“', u'"')
        t = t.replace(u'”', u'"')
        t = t.encode('ascii', 'ignore')
        return t
        
    # Tokens
                    
    def renderH(self, token):       self.f.write(u'\n\n\n### ' + token.value + u' ###\n\n\n')
    def renderMS2(self, token):     self.f.write(u'\n\n[' + token.value + u']\n\n')
    def renderP(self, token):       self.f.write(self.stopD() + self.stopNarrower() + u'\n\n    ')
    def renderB(self, token):       self.f.write(self.stopD() + self.stopNarrower() + u'\n\n    ')
    def renderS(self, token):       self.f.write(self.stopD() + self.stopNarrower() + u'\n\n    ')
    def renderS2(self, token):      self.f.write(self.stopD() + self.stopNarrower() + u'\n\n    ')
    def renderC(self, token):       self.f.write(u' ' )
    def renderV(self, token):       self.f.write(u' ' )
    def renderTEXT(self, token):    self.f.write(self.escape(token.value))
    def renderQ(self, token):       self.f.write(self.stopD() + self.startNarrower(1))
    def renderQ1(self, token):      self.f.write(self.stopD() + self.startNarrower(1))
    def renderQ2(self, token):      self.f.write(self.stopD() + self.startNarrower(2))
    def renderQ3(self, token):      self.f.write(self.stopD() + self.startNarrower(3))
    def renderNB(self, token):      self.f.write(self.stopD() + self.stopNarrower() + u"\n\n")
    def renderLI(self, token):      self.f.write(u' ')
    def renderD(self, token):       self.f.write(self.startD())
    def renderSP(self, token):      self.f.write(self.startD())
    def renderNDE(self, token):     self.f.write(u' ')
    def renderPBR(self, token):     self.f.write(u'\n')
    
    # Ignore...
    def renderXS(self,token):       self.inX = True
    def renderXE(self,token):       self.inX = False
    def renderFS(self,token):       self.inFootnote = True
    def renderFE(self,token):       self.inFootnote = False
            


# Setup list of patches and books to use
#

from __future__ import print_function, unicode_literals
import os
import logging

__logger = logging.getLogger('usfm_tools')

bookKeys = {
    'GEN': '001',
    'EXO': '002',
    'LEV': '003',
    'NUM': '004',
    'DEU': '005',
    'JOS': '006',
    'JDG': '007',
    'RUT': '008',
    '1SA': '009',
    '2SA': '010',
    '1KI': '011',
    '2KI': '012',
    '1CH': '013',
    '2CH': '014',
    'EZR': '015',
    'NEH': '016',
    'EST': '017',
    'JOB': '018',
    'PSA': '019',
    'PRO': '020',
    'ECC': '021',
    'SNG': '022',
    'ISA': '023',
    'JER': '024',
    'LAM': '025',
    'EZK': '026',
    'DAN': '027',
    'HOS': '028',
    'JOL': '029',
    'AMO': '030',
    'OBA': '031',
    'JON': '032',
    'MIC': '033',
    'NAM': '034',
    'HAB': '035',
    'ZEP': '036',
    'HAG': '037',
    'ZEC': '038',
    'MAL': '039',
    'MAT': '040',
    'MRK': '041',
    'LUK': '042',
    'JHN': '043',
    'ACT': '044',
    'ROM': '045',
    '1CO': '046',
    '2CO': '047',
    'GAL': '048',
    'EPH': '049',
    'PHP': '050',
    'COL': '051',
    '1TH': '052',
    '2TH': '053',
    '1TI': '054',
    '2TI': '055',
    'TIT': '056',
    'PHM': '057',
    'HEB': '058',
    'JAS': '059',
    '1PE': '060',
    '2PE': '061',
    '1JN': '062',
    '2JN': '063',
    '3JN': '064',
    'JUD': '065',
    'REV': '066'
}

silNames = [
    'GEN',
    'EXO',
    'LEV',
    'NUM',
    'DEU',
    'JOS',
    'JDG',
    'RUT',
    '1SA',
    '2SA',
    '1KI',
    '2KI',
    '1CH',
    '2CH',
    'EZR',
    'NEH',
    'EST',
    'JOB',
    'PSA',
    'PRO',
    'ECC',
    'SNG',
    'ISA',
    'JER',
    'LAM',
    'EZK',
    'DAN',
    'HOS',
    'JOL',
    'AMO',
    'OBA',
    'JON',
    'MIC',
    'NAM',
    'HAB',
    'ZEP',
    'HAG',
    'ZEC',
    'MAL',
    'MAT',
    'MRK',
    'LUK',
    'JHN',
    'ACT',
    'ROM',
    '1CO',
    '2CO',
    'GAL',
    'EPH',
    'PHP',
    'COL',
    '1TH',
    '2TH',
    '1TI',
    '2TI',
    'TIT',
    'PHM',
    'HEB',
    'JAS',
    '1PE',
    '2PE',
    '1JN',
    '2JN',
    '3JN',
    'JUD',
    'REV']

silNamesNTPsalms = [
    'MAT',
    'MRK',
    'LUK',
    'JHN',
    'ACT',
    'ROM',
    '1CO',
    '2CO',
    'GAL',
    'EPH',
    'PHP',
    'COL',
    '1TH',
    '2TH',
    '1TI',
    '2TI',
    'TIT',
    'PHM',
    'HEB',
    'JAS',
    '1PE',
    '2PE',
    '1JN',
    '2JN',
    '3JN',
    'JUD',
    'REV',
    'PSA']

readerNames = [
    'Gen',
    'Exod',
    'Lev',
    'Num',
    'Deut',
    'Josh',
    'Judg',
    'Ruth',
    '1Sam',
    '2Sam',
    '1Kgs',
    '2Kgs',
    '1Chr',
    '2Chr',
    'Ezra',
    'Nehm',
    'Esth',
    'Job',
    'Ps',
    'Prov',
    'Eccl',
    'Song',
    'Isa',
    'Jer',
    'Lam',
    'Ezek',
    'Dan',
    'Hos',
    'Joel',
    'Amos',
    'Obad',
    'Jonah',
    'Mic',
    'Nah',
    'Hab',
    'Zeph',
    'Hag',
    'Zech',
    'Mal',
    'Matt',
    'Mark',
    'Luke',
    'John',
    'Acts',
    'Rom',
    '1Cor',
    '2Cor',
    'Gal',
    'Eph',
    'Phil',
    'Col',
    '1Thess',
    '2Thess',
    '1Tim',
    '2Tim',
    'Titus',
    'Phlm',
    'Heb',
    'Jas',
    '1Pet',
    '2Pet',
    '1John',
    '2John',
    '3John',
    'Jude',
    'Rev']

bookNames = [
    'Genesis',
    'Exodus',
    'Leviticus',
    'Numbers',
    'Deuteronomy',
    'Joshua',
    'Judges',
    'Ruth',
    '1 Samuel',
    '2 Samuel',
    '1 Kings',
    '2 Kings',
    '1 Chronicles',
    '2 Chronicles',
    'Ezra',
    'Nehemiah',
    'Esther',
    'Job',
    'Psalms',
    'Proverbs',
    'Ecclesiastes',
    'Song of Solomon',
    'Isaiah',
    'Jeremiah',
    'Lamentations',
    'Ezekiel',
    'Daniel',
    'Hosea',
    'Joel',
    'Amos',
    'Obadiah',
    'Jonah',
    'Micah',
    'Nahum',
    'Habakkuk',
    'Zephaniah',
    'Haggai',
    'Zechariah',
    'Malachi',
    'Matthew',
    'Mark',
    'Luke',
    'John',
    'Acts',
    'Romans',
    '1 Corinthians',
    '2 Corinthians',
    'Galatians',
    'Ephesians',
    'Philippians',
    'Colossians',
    '1 Thessalonians',
    '2 Thessalonians',
    '1 Timothy',
    '2 Timothy',
    'Titus',
    'Philemon',
    'Hebrews',
    'James',
    '1 Peter',
    '2 Peter',
    '1 John',
    '2 John',
    '3 John',
    'Jude',
    'Revelation']

books = bookNames


# noinspection PyPep8Naming
def readerName(num):
    return readerNames[int(num) - 1]


# noinspection PyPep8Naming
def fullName(num):
    return bookNames[int(num) - 1]


# noinspection PyPep8Naming,PyUnusedLocal
def nextChapter(bookNumber, chapterNumber):
    return 1, 1


# noinspection PyPep8Naming
def previousChapter(bookNumber, chapterNumber):
    if chapterNumber > 1:
        return bookNumber, chapterNumber - 1
    else:
        if bookNumber > 1:
            return bookNumber - 1, 50  # bookSize[bookNumber -1])
        else:
            return 1, 1


# noinspection PyPep8Naming
def bookKeyForIdValue(book_id):
    e = book_id.find(' ')
    i = book_id if e == -1 else book_id[:e]
    return bookKeys[i]


# noinspection PyPep8Naming
def bookID(usfm):
    s = usfm.find('\id ') + 4
    e = usfm.find(' ', s)
    e2 = usfm.find('\n', s)
    e = e if e < e2 else e2
    return usfm[s:e].strip()


# noinspection PyPep8Naming
def bookName(usfm):
    book_id = bookID(usfm)
    index = silNames.index(book_id)
    return bookNames[index]


# noinspection PyPep8Naming
def loadBooks(path):
    loaded_books = {}
    dirList = os.listdir(path)
    __logger.info('LOADING ALL USFM FILES FROM ' + path)
    for fname in dirList:

        full_file_name = os.path.join(path, fname)
        if not os.path.isfile(full_file_name):
            continue

        if fname[-4:].lower() in ['.pdf', '.sig']:
            continue

        # noinspection PyBroadException
        try:
            f = open(full_file_name, 'U')  # U handles line endings
            usfm = f.read().decode('utf-8-sig').lstrip()
            if usfm[:4] == r'\id ' and usfm[4:7] in silNames:
                # print '     Loaded ' + fname + ' as ' + usfm[4:7]
                loaded_books[bookID(usfm)] = usfm
                f.close()
            else:
                __logger.info('Ignored ' + fname)
        except:
            __logger.warning("Couldn't open " + fname)
    __logger.info('FINISHED LOADING\n')
    return loaded_books


# noinspection PyPep8Naming
def orderFor(booksDict):
    order = silNames
    if 'PSA' in booksDict and 'GEN' not in booksDict and 'MAT' in booksDict:
        # This is a big hack. When doing Psalms + NT, put Psalms last
        order = silNamesNTPsalms
    a = []
    for book_name in order:
        if book_name in booksDict:
            a.append(booksDict[book_name])
    return a



# -*- coding: utf-8 -*-
#

from .abstractRenderer import AbstractRenderer
import codecs
import datetime

#
#   Renders to ConTeXt so we can make PDF. Main renderer for PDF of OEB
#

class ConTeXtRenderer(AbstractRenderer):

    def __init__(self, inputDir, outputFilename):
        # Unset
        self.f = None  # output file stream
        # IO
        self.outputFilename = outputFilename
        self.inputDir = inputDir
        # Flags
        self.printerState = {u'li': False, u'd': False}
        self.smallCapSections = True  # Sometimes we don't want to do this, like for Psalms
        self.justDidLORD = False
        self.justDidNB = False
        self.doNB = False
        self.narrower = False
        self.doChapterOrVerse = u''
        self.smallcaps = False

    def render(self):
        self.f = codecs.open(self.outputFilename, 'w', 'utf_8_sig')
        self.loadUSFM(self.inputDir)
        self.f.write(u"""
            Document rendered on """ + datetime.date.today().strftime("%F") + r"""
            \par
            \page[right]
            \par ~
            {\midaligned {\tfc{\WORD{Table of Contents}}}}
            \par ~
            \placelist[chapter]
        """)
        self.run()
        self.f.write(self.stopNarrower() + self.closeTeXt)
        self.f.close()

    def writeLog(self, s):
        print(s.encode('ascii', 'ignore'))


    #
    #   Support
    #

    def startNarrower(self, n):
        s = u'}' if self.narrower else u'\n\\blank[medium] '
        self.narrower = True
        s = s + u'\n\\noindentation \\Q{' + str(n) + u'}{'
        self.doNB = True
        return s

    def stopNarrower(self):
        s = u'}\n\\blank[medium] ' if self.narrower else u''
        self.narrower = False
        return s

    def escapeText(self, s):
        return ( s.replace(u'&', u'\\&').replace(u'%', u'\\%')
                  .replace(u'#', u'\\#').replace(u'$', u'\\$')
                  .replace(u'_', u'\\_').replace(u'{', u'\\{')
                  .replace(u'}', u'\\}') )

    def markForSmallCaps(self):
        if self.smallCapSections:
             self.smallcaps = True

    def renderSmallCaps(self, s):
        if self.smallcaps == True:
            self.smallcaps = False
            return self.smallCapText(s)
        return s

    def smallCapText(self, s):
         i = 0
         while i < len(s):
             if i < 50:  #we are early, look for comma
                 if s[i] == u',' or s[i] == u';' or s[i] == u'(' or s[i:i+3] == u'and':
                     return u'{\sc ' + s[:i+1] + u'}' + s[i+1:]
             else: # look for space
                 if s[i] == ' ':
                     return u'{\sc ' + s[:i] + u'}' + s[i:]
             i = i + 1
         return u'{\sc ' + s + u'}'

    def startLI(self):
        if self.printerState[u'li'] == False:
            self.printerState[u'li'] = True
            #return u'\startitemize \item '
            return r'\startexdent '
        else:
            #return u'\item '
            return r'\par '

    def stopLI(self):
        if self.printerState[u'li'] == False:
            return u''
        #elif self.printerState[u'li2'] == False:
            #return u''
        #elif self.printerState[u'li3'] == False:
            #return u''
        else:
            self.printerState[u'li'] = False
            #return u'\stopitemize'
            return r'\stopexdent '

    def startD(self):
        if self.printerState[u'd'] == False:
            self.printerState[u'd'] = True
        return u'\par {\startalignment[center] \em '

    def stopD(self):
        if self.printerState[u'd'] == False:
            return u''
        else:
            self.printerState[u'd'] = False
            return u'\stopalignment }'

    def newLine(self):
        s = u'\n\par \n'
        if self.doNB:
            self.doNB = False
            self.justDidNB = True
            s = s + r'\noindentation '
        elif self.justDidNB:
            self.justDidNB = False
            s = s + r'\indentation '
        return s


    #
    #   Tokens
    #

    def renderID(self, token):      self.f.write( self.stopNarrower() + r"\marking[RAChapter]{ } \marking[RABook]{ } \marking[RASection]{ }" )
    def renderH(self, token):       self.f.write( u'\n\n\RAHeader{' + self.escapeText(token.value) + u'}\n')
    def renderTOC1(self, token):      self.f.write( self.stopLI() + self.stopNarrower() + u'\n\TOC1{' + token.value + u'}\n')
    def renderTOC2(self, token):      self.f.write( self.stopLI() + self.stopNarrower() + u'\n\TOC2{' + token.value + u'}\n')
    def renderTOC3(self, token):      self.f.write( self.stopLI() + self.stopNarrower() + u'\n\TOC3{' + token.value + u'}\n')
    def renderMT(self, token):      self.f.write( self.stopLI() + self.stopNarrower() + u'\n\MT{' + token.value + u'}\n')
    def renderMT2(self, token):     self.f.write( self.stopLI() + self.stopNarrower() + u'\n\MTT{' + token.value + u'}\n')
    def renderMS(self, token):      self.markForSmallCaps() ; self.f.write(self.stopNarrower() + u'\n\MS{' + self.escapeText(token.value) + u'}\n') ; self.doNB = True
    def renderMS2(self, token):     self.doNB = True; self.markForSmallCaps() ; self.f.write( self.stopNarrower() + u'\n\MSS{' + self.escapeText(token.value) + '}' + self.newLine() )
    def renderP(self, token):       self.f.write( self.stopD() + self.stopLI() + self.stopNarrower() + self.newLine() )
    def renderB(self, token):       self.f.write( self.stopD() + self.stopLI() + self.stopNarrower() + u'\\blank \n' )
    def renderS(self, token):       self.f.write( self.stopD() + self.stopLI() + self.stopNarrower() +  u'\n\\blank[big] ' + u'\n\MSS{' + self.escapeText(token.value) + '}' + self.newLine() ) ; self.doNB = True
    def renderS2(self, token):      self.doNB = True; self.f.write( self.stopD() + self.stopLI() + self.stopNarrower() + u'\n\\blank[big] ' + u'\n\MSS{' + self.escapeText(token.value) + '}' + self.newLine() )
    def renderS5(self, token):      self.doNB = True; self.f.write( self.stopD() + self.stopLI() + self.stopNarrower() + u'\n\\blank[big] ' + u'\n\MSS{' + self.escapeText(token.value) + '}' + self.newLine() )
    def renderC(self, token):
        self.doChapterOrVerse = u'\C{' + self.escapeText(token.value) + u'}'
        self.f.write( u' ' )
    def renderV(self, token):
        if not token.value == u'1':
            self.doChapterOrVerse =  u'\V{' + self.escapeText(token.value) + u'}'
        self.f.write( ' ' )
    def renderWJS(self, token):     self.f.write( u" " )
    def renderWJE(self, token):     self.f.write( u" " )
    def renderTEXT(self, token):
        s = self.escapeText(token.value)
        if self.smallcaps and not self.doChapterOrVerse == u'':
            s = self.renderSmallCaps(s)
            s = self.doChapterOrVerse + s
            self.doChapterOrVerse = u''
        elif not self.doChapterOrVerse == u'':
            i = s.find(u' ')
            if i == -1:
                # No space found - try end
                i = len(s)
            s = s[:i] + self.doChapterOrVerse + s[i+1:]
            self.doChapterOrVerse = u''
        elif self.smallcaps:
            s = self.renderSmallCaps(s)
        if self.justDidLORD:
            if s[0].isalpha():
                s = u' ' + s
            self.justDidLORD = False
        self.f.write(s)
        self.f.write(u' ')
    def renderQ(self, token):       self.renderQ1(token)
    def renderQ1(self, token):      self.f.write( self.stopD() + self.stopLI() + self.startNarrower(1) )
    def renderQ2(self, token):      self.f.write( self.stopD() + self.stopLI() + self.startNarrower(2) )
    def renderQ3(self, token):      self.f.write( self.stopD() + self.stopLI() + self.startNarrower(3) )
    def renderNB(self, token):      self.doNB = True ; self.f.write( self.stopD() + self.stopLI() + self.stopNarrower() + u'\\blank[medium] ' + self.newLine() )
    def renderFS(self, token):      self.f.write( u'\\footnote{' )
    def renderFE(self, token):      self.f.write( u'} ' )
    def renderFP(self, token):      self.f.write( self.newLine() )
    def renderIS(self, token):      self.f.write( u'{\em ' )
    def renderIE(self, token):      self.f.write( u'} ' )
    def renderBDS(self, token):     self.f.write( u'{\\bf ')
    def renderBDE(self, token):     self.f.write( u'} ')
    def renderBDITS(self, token):   self.f.write( u'{\\bs ')
    def renderBDITE(self, token):   self.f.write( u'} ')
    def renderADDS(self, token):    self.f.write( u'{\em ' )
    def renderADDE(self, token):    self.f.write( u'} ' )
    def renderNDS(self, token):     self.f.write( u'{\sc ' )
    def renderNDE(self, token):     self.justDidLORD = True; self.f.write( u'}' )
    def renderLI(self, token):      self.f.write( self.startLI() )
    def renderLI1(self, token):      self.f.write( self.startLI() )
    def renderLI2(self, token):      self.f.write( self.startLI() )
    def renderLI3(self, token):      self.f.write( self.startLI() )
    def renderD(self, token):       self.f.write( self.startD() )
    def renderSP(self, token):      self.f.write( self.startD() )
    def renderPBR(self, token):     self.f.write( u' \\\\ ' )
    def renderFR(self, token):      self.f.write( u' ' + self.escapeText(token.value) + u' ' )
    def renderFRE(self, token):     self.f.write( u' ' )
    def renderFK(self, token):      self.f.write( u' ' + self.escapeText(token.value) + u' ' )
    def renderFT(self, token):      self.f.write( u' ' + self.escapeText(token.value) + u' ' )
    def renderPI(self, token):      self.renderQ(token)

    def renderQSS(self, token):      return
    def renderQSE(self, token):      return

    def render_is1(self, token):    self.renderS(token)
    def render_imt1(self, token):   self.f.write( self.stopLI() + self.stopNarrower() + u'\n\IMT1{' + token.value + u'}\n')
    def render_imt2(self, token):   self.f.write( self.stopLI() + self.stopNarrower() + u'\n\IMT2{' + token.value + u'}\n')
    def render_imt3(self, token):   self.f.write( self.stopLI() + self.stopNarrower() + u'\n\IMT3{' + token.value + u'}\n')
    def render_ip(self, token):     self.renderP(token)
    def render_iot(self, token):    self.renderQ(token)
    def render_io1(self, token):    self.renderQ2(token)
    def render_io2(self, token):    self.renderQ2(token)

    closeTeXt = r"""
    \stoptext
    """



# -*- coding: utf-8 -*-
#

from .abstractRenderer import AbstractRenderer
import codecs
import datetime
from .books import bookKeyForIdValue

from .parseUsfm import UsfmToken

#
#   Simplest renderer. Ignores everything except ascii text.
#

class SingleHTMLRenderer(AbstractRenderer):
    def __init__(self, inputDir, outputFilename):
        # Unset
        self.f = None  # output file stream
        # IO
        self.outputFilename = outputFilename
        self.inputDir = inputDir
        # Position
        self.cb = u''       # Current Book
        self.cc = u'001'    # Current Chapter
        self.cv = u'001'    # Current Verse
        self.indentFlag = False
        self.bookName = u''
        self.chapterLabel = u'Chapter'
        self.listItemLevel = 0
        self.footnoteFlag = False
        self.fqaFlag = False
        self.footnotes = {}
        self.footnote_id = u''
        self.footnote_num = 1
        self.footnote_text = u''
        self.paragraphOpen = False

    def render(self):
        self.loadUSFM(self.inputDir)
        self.f = codecs.open(self.outputFilename, 'w', 'utf_8_sig')
        self.writeHeader()
        self.run()
        self.write(self.stopIndent())
        self.write(self.stopLI())
        self.write(self.stopP())
        self.writeFootnotes()
        self.writeClosing()
        self.f.close()

    def writeHeader(self):
        h = u"""
<!DOCTYPE html PUBLIC "-//W3C//DTD XHTML 1.0 Strict//EN" "http://www.w3.org/TR/xhtml1/DTD/xhtml1-strict.dtd">
<html lang="en" xmlns="http://www.w3.org/1999/xhtml">
<head>
    <meta http-equiv="content-type" content="text/html; charset=utf-8"></meta>
    <title>Bible</title>
    <style media="all" type="text/css">
        .indent-0 {
            margin-left:0em;
            margin-bottom:0em;
            margin-top:0em;
        }
        .indent-1 {
            margin-left:0em;
            margin-bottom:0em;
            margin-top:0em;
        }
        .indent-2 {
            margin-left:1em;
            margin-bottom:0em;
            margin-top:0em;
        }
        .indent-3 {
            margin-left:2em;
            margin-bottom:0em;
            margin-top:0em;
        }
        .c-num {
            color:gray;
        }
        .v-num {
            color:gray;
        }
        .tetragrammaton {
            font-variant: small-caps;
        }
        .footnotes {
            font-size: 0.8em;
        }
        .footnotes-hr {
            width: 90%;
        }
    </style>
</head>
<body>
"""
        self.write(h)

    def writeClosing(self):
        h = """
        </div>
    </body>
</html>
"""
        self.write(h)

    def startLI(self, level=1):
        if self.listItemLevel:
            self.stopLI()
        ret = u''
        self.listItemLevel = 0
        while self.listItemLevel < level:
            ret += u'<ul>'
            self.listItemLevel += 1 
        return ret

    def stopLI(self):
        ret = u''
        while self.listItemLevel > 0:
            ret += u'</ul>'
            self.listItemLevel -= 1
        return ret

    def stopP(self):
        if self.paragraphOpen:
            self.paragraphOpen = False
            return u'\n</p>\n\n'
        return u''

    def escape(self, s):
        return s.replace(u'~',u'&nbsp;')

    def write(self, unicodeString):
        self.f.write(unicodeString.replace(u'~', u' '))

    def writeIndent(self, level):
        if self.indentFlag:
            self.write(self.stopIndent())  # always close the last indent before starting a new one
        if level > 0:
            self.indentFlag = True
            self.write(self.stopP())
            self.write(u'\n<p class="indent-' + str(level) + u'">\n')
            self.write(u'&nbsp;' * (level * 4))  # spaces for PDF since we can't style margin with css

    def stopIndent(self):
        if self.indentFlag:
            self.indentFlag = False
            return u'\n</p>\n'
        else:
            return u''

    def renderID(self, token):
        self.write(self.stopIndent())
        self.write(self.stopLI())
        self.write(self.stopP())
        self.writeFootnotes()
        if self.cb:
            self.write('</div>\n\n');
        self.bookName = None
        self.cb = bookKeyForIdValue(token.value)
        self.chapterLabel = u'Chapter'
        self.write('<div id="bible-book-'+self.cb+'" class="bible-book-text">\n')

    def renderH(self, token):
        self.bookName = token.value
        self.write('<h1>' + self.bookName + u'</h1>\n')

    def renderTOC2(self, token):
        if not self.bookName:
            self.renderH(token)

    def renderMT(self, token):
        return  #self.write(u'\n\n<h1>' + token.value + u'</h1>') # removed to use TOC2

    def renderMT2(self, token):
        self.write(u'\n\n<h2>' + token.value + u'</h2>\n')

    def renderMT3(self, token):
        self.write(u'\n\n<h2>' + token.value + u'</h2>\n')

    def renderMS1(self, token):
        self.write(u'\n\n<h3>' + token.value + u'</h3>\n')

    def renderMS2(self, token):
        self.write(u'\n\n<h4>' + token.value + u'</h4>\n')

    def renderP(self, token):
        self.write(self.stopIndent())
        self.write(self.stopLI())
        self.write(self.stopP())
        self.write(u'\n\n<p>\n')
        self.paragraphOpen = True

    def renderPI(self, token):
        self.write(self.stopIndent())
        self.write(self.stopLI())
        self.write(self.stopP())
        self.writeIndent(2)

    def renderM(self, token):
        self.write(self.stopIndent())
        self.write(self.stopLI())
        self.write(self.stopP())
        self.write(u'\n\n<p>\n')
        self.paragraphOpen = True

    def renderS1(self, token):
        self.write(self.stopIndent())
        self.write(self.stopLI())
        self.write(u'\n\n<h4 style="text-align:center">' + token.getValue() + u'</h4>')

    def renderS2(self, token):
        self.write(self.stopIndent())
        self.write(self.stopLI())
        self.write(u'\n\n<h5 style="text-align:center">' + token.getValue() + u'</h5>')

    def renderS3(self, token):
        self.write(self.stopIndent())
        self.write(self.stopLI())
        self.write(u'\n\n<h5">' + token.getValue() + u'</h5>')

    def renderC(self, token):
        self.closeFootnote()
        self.write(self.stopIndent())
        self.write(self.stopLI())
        self.write(self.stopP())
        self.writeFootnotes()
        self.footnote_num = 1
        self.cc = token.value.zfill(3)
        self.write(u'\n\n<h2 id="{0}-ch-{1}" class="c-num">{2} {3}</h2>'
            .format(self.cb, self.cc, self.chapterLabel, token.value))

    def renderV(self, token):
        self.closeFootnote()
        self.cv = token.value.zfill(3)
        self.write(self.stopLI())
        self.write(u' <span id="{0}-ch-{1}-v-{2}" class="v-num"><sup><b>{3}</b></sup></span>'
            .format(self.cb, self.cc, self.cv, token.value))

    def renderWJS(self, token):
        self.write(u'<span class="woc">')

    def renderWJE(self, token):
        self.write(u'</span>')

    def renderTEXT(self, token):
        self.write(u" " + self.escape(token.value) + u" ")

    def renderQ(self, token):
        self.writeIndent(1)

    def renderQ1(self, token):
        self.writeIndent(1)

    def renderQ2(self, token):
        self.writeIndent(2)

    def renderQ3(self, token):
        self.writeIndent(3)

    def renderNB(self, token):
        self.write(self.stopIndent())

    def renderB(self, token):
        self.write(self.stopLI())
        self.write(u'\n\n<p class="indent-0">&nbsp;</p>')

    def renderIS(self, token):
        self.write(u'<i>')

    def renderIE(self, token):
        self.write(u'</i>')

    def renderNDS(self, token):
        self.write(u'<span class="tetragrammaton">')

    def renderNDE(self, token):
        self.write(u'</span>')

    def renderPBR(self, token):
        self.write(u'<br></br>')

    def renderSCS(self, token):
        self.write(u'<b>')

    def renderSCE(self, token):
        self.write(u'</b>')

    def renderFS(self, token):
        self.closeFootnote()
        self.footnote_id = u'fn-{0}-{1}-{2}-{3}'.format(self.cb, self.cc, self.cv, self.footnote_num)
        self.write(u'<span id="ref-{0}"><sup><i>[<a href="#{0}">{1}</a>]</i></sup></span>'.format(self.footnote_id, self.footnote_num))
        self.footnoteFlag = True
        text = token.value
        if text.startswith(u'+ '):
            text = text[2:]
        elif text.startswith(u'+'):
            text = text[1:]
        self.footnote_text = text

    def renderFT(self, token):
        self.footnote_text += token.value

    def renderFE(self, token):
        self.closeFootnote()

    def renderFP(self, token):
        self.write(u'<br />')

    def renderQSS(self, token):
        self.write(u'<i class="quote selah" style="float:right;">')

    def renderQSE(self, token):
        self.write(u'</i>')

    def renderEMS(self, token):
        self.write(u'<i class="emphasis">')

    def renderEME(self, token):
        self.write(u'</i>')

    def renderE(self, token):
        self.write(self.stopIndent())
        self.write(self.stopP())
        self.write(u'\n\n<p>' + token.value + '</p>')

    def renderPB(self, token):
        pass

    def renderPERIPH(self, token):
        pass

    def renderLI(self, token):
        self.renderLI1(token)

    def renderLI1(self, token):
        self.write(self.startLI(1))

    def renderLI2(self, token):
        self.write(self.startLI(2))

    def renderLI3(self, token):
        self.write(self.startLI(3))

    def renderS5(self, token):
        self.write(u'\n<span class="chunk-break"></span>\n')

    def render_imt1(self, token):
        self.write(u'\n\n<h2>' + token.value + u'</h2>')

    def render_imt2(self, token):
        self.write(u'\n\n<h3>' + token.value + u'</h3>')

    def render_imt3(self, token):
        self.write(u'\n\n<h4>' + token.value + u'</h4>')

    def renderCL(self, token):
        self.chapterLabel = token.value

    def renderQR(self, token):
        self.write(u'<i class="quote right" style="display:block;float:right;">'+token.value+'</i>')

    def renderFQA(self, token):
        self.footnote_text += u'<i>'+token.value
        self.fqaFlag = True

    def renderFQAE(self, token):
        if self.fqaFlag:
            self.footnote_text += u'</i>'+token.value
        self.fqaFlag = False

    def closeFootnote(self):
        if self.footnoteFlag:
            self.footnoteFlag = False
            self.renderFQAE(UsfmToken(u''))
            self.footnotes[self.footnote_id] = {
                'text': self.footnote_text,
                'book': self.cb,
                'chapter': self.cc,
                'verse': self.cv,
                'footnote': self.footnote_num
            }
            self.footnote_num += 1
            self.footnote_text = u''
            self.footnote_id = u''

    def writeFootnotes(self):
        fkeys = self.footnotes.keys()
        if len(fkeys) > 0:
            self.write(u'<div class="footnotes">')
            self.write(u'<hr class="footnotes-hr"/>')
            for fkey in sorted(fkeys):
                footnote = self.footnotes[fkey]
                self.write(u'<div id="{0}" class="footnote">{1}:{2} <sup><i>[<a href="#ref-{0}">{5}</a>]</i></sup><span class="text">{6}</span></div>'
                    .format(fkey, footnote['chapter'].lstrip('0'), footnote['verse'].lstrip('0'), footnote['chapter'], footnote['verse'], footnote['footnote'], footnote['text']))
            self.write(u'</div>')
        self.footnotes = {}

    def renderQA(self, token):
        self.write(u'<p class="quote acrostic heading" style="text-align:center;text-style:italic;">'+token.value+u'</p>')

    def renderQAC(self, token):
        self.write(u'<i class="quote acrostic character">')

    def renderQACE(self,token):
        self.write(u'</i>')




# -*- coding: utf-8 -*-
#

import os
from .parseUsfm import parseString
from .books import bookKeyForIdValue, loadBooks, silNames

class DummyFile(object):
    def close(self):
        pass
    def write(self, str):
        pass

class MediaWikiPrinter(object):
    def __init__(self, outputDir):
        self.outputDir = outputDir
        self.f = DummyFile()
        self.cb = u''       # Current Book
        self.cc = u'001'    # Current Chapter
        self.ccUnfil = u'1' # same, not padded.
        self.cv = u'001'    # Currrent Verse
        self.indentFlag   = False
        self.footnoteFlag = False

    def write(self, unicodeString):
        self.f.write(unicodeString.encode('utf-8'))

    def renderID(self, token):
        self.write(u'</p>')
        self.f.close()
        self.cb = bookKeyForIdValue(token.value)
        self.f = open(self.outputDir + u'/c' + self.cb + u'001.html', 'w')
        self.write(u'\n<!-- \\id ' + self.cb + u' -->')
        self.indentFlag = False
    def renderIDE(self, token):     pass
    def renderSTS(self, token):     pass
    def renderTOC2(self, token):    self.write(u' Bible:' + token.value + u'_# ')
    def renderH(self, token):       self.write(u'\n<!-- \\h ' + token.value + u' -->')
    def renderMT(self, token):      self.write(u'\n<!-- \\mt1 ' + token.value + u' -->')
    def renderMS(self, token):      pass
    def renderMS2(self, token):     pass
    def renderP(self, token):
        self.indentFlag = False
        self.write(u'\n\n')
    def renderS(self, token):
        self.indentFlag = False
        self.write(u'\n=== ' + token.value + u' === ')
    def renderS2(self, token):
        self.indentFlag = False
        pass
    def renderR(self, token):       self.write('\n<span class="srefs">' + token.value + '</span>')
    def renderC(self, token):
        self.cc = token.value.zfill(3)
        self.ccUnfil = token.getValue()
        if self.cc == u'001':
            self.write(u'Bible:' + self.cb + u'_' + token.value + u' ')
        else:
            self.write(u'\n\n')
            self.f.close()
            self.f = open(self.outputDir + u'/c' + self.cb + self.cc + u'.html', 'w')
            self.write(u'Bible:' + self.cb + u'_' + token.value + u' ')
    def renderV(self, token):
        self.cv = token.value.zfill(3)
        if not self.cv == u'001': self.write(u'<\span>\n')
        self.write(u'<span id="' + self.ccUnfil + u'_' + token.getValue() + u'"><sup>' + token.getValue() + u'</sup>')

    def renderWJS(self, token):     pass
    def renderWJE(self, token):     pass
    def renderTEXT(self, token):
        if self.footnoteFlag:       self.footnoteFlag = False; self.write(u' -->')
        self.write(token.getValue())
    def renderQ(self, token):       self.write(u'\n:')
    def renderQ1(self, token):      self.write(u'\n::')
    def renderQ2(self, token):      pass
    def renderQ3(self, token):      pass
    def renderNB(self, token):      pass
    def renderQTS(self, token):     pass
    def renderQTE(self, token):     pass
    def renderFS(self, token):      self.write(u'<ref><!-- '); self.footnoteFlag = True
    def renderFT(self, token):      self.write(u'\\ft ' + token.getValue())
    def renderFK(self, token):      self.write(u'\\fk ' + token.getValue())
    def renderFE(self, token):
        if self.footnoteFlag:       self.footnoteFlag = False; self.write(u' -->')
        self.write(u'</ref>')
    def renderFP(self, token):
        self.indentFlag = False
        self.write(u'\n\n')
    def renderIS(self, token):      pass
    def renderIE(self, token):      pass
    def renderB(self, token):       pass
    def renderD(self, token):       pass
    def renderADDS(self, token):    pass
    def renderADDE(self, token):    pass
    def renderLI(self, token):      self.write(u'\n:<!-- \li -->')
    def renderSP(self, token):      pass
    def renderNDS(self, token):     pass
    def renderNDE(self, token):     pass
    def renderPBR(self, token):     pass
    def renderFR(self, token):      pass
    def renderFRE(self, token):     pass
    def renderXS(self, token):      pass
    def renderXE(self, token):      pass
    def renderXDCS(self, token):    pass
    def renderXDCE(self, token):    pass
    def renderXO(self, token):      pass
    def renderXT(self, token):      pass
    def renderM(self, token):       pass
    def renderMI(self, token):      pass
    def renderTLS(self, token):     pass
    def renderTLE(self, token):     pass
    def renderPI(self, token):      pass
    def renderSCS(self, token):     pass
    def renderSCE(self, token):     pass
    def renderD(self, token):       pass # For now
    def renderREM(self, token):     pass # This is for comments in the USFM

class Transform(object):

    def stripUnicodeHeader(self, unicodeString):
        if unicodeString[0] == u'\ufeff':
            return unicodeString[1:]
        else:
            return unicodeString

    def translateBook(self, usfm):
         tokens = parseString(usfm)
         tp = MediaWikiPrinter(self.outputDir)
         for t in tokens: t.renderOn(tp)

    def setupAndRun(self, patchedDir, outputDir):
        self.outputDir = outputDir
        self.booksUsfm = loadBooks(patchedDir)

        for bookName in silNames:
            if self.booksUsfm.has_key(bookName):
                print('     ' + bookName)
                self.translateBook(self.booksUsfm[bookName])



import logging

# set up logging
__logger = logging.getLogger('usfm_tools')
__logger.setLevel(logging.INFO)
__handler = logging.StreamHandler()
__handler.setLevel(logging.INFO)
__formatter = logging.Formatter("%(asctime)s %(name)s [%(levelname)s] %(message)s")
__handler.setFormatter(__formatter)
__logger.addHandler(__handler)


from setuptools import setup

setup(
    name="usfm_tools",
    version="0.0.30",
    author="unfoldingWord",
    author_email="info@unfoldingWord.org",
    description="A framework for transforming .usfm files into specified targets",
    license="MIT",
    keywords="unfoldingWord usfm tools",
    url="https://github.com/unfoldingWord-dev/USFM-Tools",
    packages=['usfm_tools', 'usfm_tools/support'],
    long_description='This project comprises a framework for transforming .usfm files into specified targets. It is '
                     'primarily used for the Open English Bible, and may need adjustment if used for other purposes. '
                     'This fork of USFM-Tools includes basic support for conversion to USX.',
    classifiers=[],
    install_requires=[
        'future==0.16.0',
        'pyparsing==2.1.10'
    ]
)



# -*- coding: utf-8 -*-
#

from .abstractRenderer import AbstractRenderer
import codecs
from .books import bookKeyForIdValue

#
#   Renders as set of web pages
#

class DummyFile(object):
    def close(self):
        pass
    def write(self, str):
        pass

class HTMLRenderer(AbstractRenderer):

    def __init__(self, inputDir, outputDir, oebFlag=False):
        # Unset
        self.f = DummyFile()  # output file stream
        self.ft = [] # array of text to write to file
        # IO
        self.outputDir = outputDir
        self.inputDir = inputDir
        # Caches
        self.cachedChapterMarker = u''
        self.cachedBookname = u''
        # Position
        self.cb = u''    # Current Book
        self.cc = u'001'    # Current Chapter
        self.cv = u'001'    # Currrent Verse
        # Flags
        self.indentFlag = False
        self.oebFlag = oebFlag
        self.fileCounter = 0
        self.secondaryCounter = 0

    def render(self):
        self.f = DummyFile()
        # Write pages
        self.loadUSFM(self.inputDir)
        self.run()
        self.close()

    def writeLog(self, s):
        print(s)

    # File handling

    def openFile(self, bookID):
        self.f = open(self.outputDir + u'/b' + bookID + u'_' + str(self.fileCounter) + u'.html', 'w')
        self.bookID = bookID
        self.ft = []

    def close(self):
        t = u''.join(self.ft)
        self.f.write(self.cleanHTML(t).encode('utf-8'))
        self.f.close()

    def write(self, unicodeString):
        self.ft.append(unicodeString)

    def cleanHTML(self, t):
        c = t
        c = t.replace(u'<p><br /><br />', u'<p>')
        c = c.replace(r'~', u'&nbsp;')
        if self.oebFlag:
            c = c.replace(r'%navmarker%', u'OEB')
            c = c.replace(r'%linkToWebsite%',u'<tr><td colspan = "2"><a href="http://openenglishbible.org">OpenEnglishBible.org</a></td></tr>')
        else:
            c = c.replace(r'%navmarker%', u'<div style="font-size:200%;color:green;">✝</div>')
            c = c.replace(r'%linkToWebsite%',u'')
        return c

    # Support
    def incrementFile(self):
        self.ft.append(footer)
        self.ft.append(u'</html>')
        self.close()
        self.ft = []
        self.ft.append(header)
        self.ft.append("<p>")
        self.fileCounter +=1
        self.f = open(self.outputDir + u'/b' + self.bookID + u'_' + str(self.fileCounter) + u'.html', 'w')

    def writeChapterMarker(self):
        self.write(self.cachedChapterMarker)
        self.cachedChapterMarker = u''

    def writeIndent(self, level):
        if level == 0:
            self.indentFlag = False
            self.write(u'<br /><br />')
            return
        if not self.indentFlag:
            self.indentFlag = True
            self.write(u'<br />')
        self.write(u'<br />')
        self.write(u'&nbsp;&nbsp;' * level)
        self.writeChapterMarker()

    def renderID(self, token):
        self.write(footer)
        self.close()
        self.cb = bookKeyForIdValue(token.value)
        self.openFile(self.cb)
        self.write(header)
        self.indentFlag = False
    def renderTOC2(self, token):      self.write(u'</p><h2>' + token.value + u'</h2><p>')
    def renderMT(self, token):      self.write(u'</p><h1>' + token.value + u'</h1><p>')
    def renderMT2(self, token):      self.write(u'</p><h2>' + token.value + u'</h2><p>')
    def renderMS(self, token):      self.write(u'</p><h4>' + token.value + u'</h4><p>')
    def renderMS2(self, token):     self.write(u'</p><h5>' + token.value + u'</h5><p>')
    def renderP(self, token):
        self.indentFlag = False
        self.write(u'<br /><br />')
        self.writeChapterMarker()
    def renderS(self, token):
        self.indentFlag = False
        if token.value == u'~':
            self.write(u'<p>&nbsp;</p><p>')
        else:
            self.write(u'</p><h6>' + token.value + u'</h6><p>')
    def renderS2(self, token):
        self.indentFlag = False
        self.write(u'</p><h7>' + token.value + u'</h7><p>')
    def renderS5(self,token):
        if (self.secondaryCounter>=5):
            self.secondaryCounter = 1
            self.incrementFile()
        else:
            self.secondaryCounter+=1
    def renderC(self, token):
        self.cc = token.value.zfill(3)
        self.cachedChapterMarker = u'<span class="chapter">' + token.value + u'</span>'
        # if self.cb==u'019': self.write(u'<p><em>Psalm ' + token.value + u'</em></p>')
    def renderV(self, token):
        self.cv = token.value.zfill(3)
        if self.cv == u'001':
            pass
        else:
            self.write(u'\n<span class="verse" rel="v' + self.cb + self.cc + self.cv + u'">' + token.value + u'</span>\n')
    def renderWJS(self, token):     self.write(u'<span class="woc">')
    def renderWJE(self, token):     self.write(u'</span>')

    def renderNDS(self, token):     self.write(u'<span class="nd">')
    def renderNDE(self, token):     self.write(u'</span>')

    def renderTEXT(self, token):    self.write(u" " + token.value + u" ")
    def renderQ(self, token):       self.writeIndent(1)
    def renderQ1(self, token):      self.writeIndent(1)
    def renderQ2(self, token):      self.writeIndent(2)
    def renderQ3(self, token):      self.writeIndent(3)
    def renderNB(self, token):      self.writeIndent(0)
    def renderB(self, token):       self.write(u'<br />')
    def renderIS(self, token):      self.write(u'<i>')
    def renderIE(self, token):      self.write(u'</i>')
    def renderBDS(self, token):     self.f.write(u'<b>')
    def renderBDE(self, token):     self.f.write(u'</b>')
    def renderBDITS(self, token):   self.f.write(u'<b><i>')
    def renderBDITE(self, token):   self.f.write(u'</b></i>')
    def renderPBR(self, token):     self.write(u'<br />')

    #handle tables
    def renderTR(self,token):
        self.write(u'<tr>' + token.value + '</tr>')
    def renderTHR1(self,token):
        self.write(u'<th class="align-right">' + token.value + '</th>')
    def renderTHR2(self,token):
        self.write(u'<th class="align-right">' + token.value + '</th>')
    def renderTHR3(self,token):
        self.write(u'<th class="align-right">' + token.value + '</th>')
    def renderTHR4(self,token):
        self.write(u'<th class="align-right">' + token.value + '</th>')
    def renderTHR5(self,token):
        self.write(u'<th class="align-right">' + token.value + '</th>')
    def renderTHR6(self,token):
        self.write(u'<th class="align-right">' + token.value + '</th>')

    def renderTH1(self,token):
        self.write(u'<th>' + token.value + '</th>')
    def renderTH2(self,token):
        self.write(u'<th>' + token.value + '</th>')
    def renderTH3(self,token):
        self.write(u'<th>' + token.value + '</th>')
    def renderTH4(self,token):
        self.write(u'<th>' + token.value + '</th>')
    def renderTH5(self,token):
        self.write(u'<th>' + token.value + '</th>')
    def renderTH6(self,token):
        self.write(u'<th>' + token.value + '</th>')
    #table column right aligned
    def renderTCR1(self,token):
        self.write(u'<td class="align-right">' + token.value + '</td>')
    def renderTCR2(self,token):
        self.write(u'<td class="align-right">' + token.value + '</td>')
    def renderTCR3(self,token):
        self.write(u'<td class="align-right">' + token.value + '</td>')
    def renderTCR4(self,token):
        self.write(u'<td class="align-right">' + token.value + '</td>')
    def renderTCR5(self,token):
        self.write(u'<td class="align-right">' + token.value + '</td>')
    def renderTCR6(self,token):
        self.write(u'<td class="align-right">' + token.value + '</td>')

    #table column
    def renderTC1(self,token):
        self.write(u'<td>' + token.value + '</td>')
    def renderTC2(self,token):
        self.write(u'<td>' + token.value + '</td>')
    def renderTC3(self,token):
        self.write(u'<td>' + token.value + '</td>')
    def renderTC4(self,token):
        self.write(u'<td>' + token.value + '</td>')
    def renderTC5(self,token):
        self.write(u'<td>' + token.value + '</td>')
    def renderTC6(self,token):
        self.write(u'<td>' + token.value + '</td>')

    def renderD(self, token):       self.writeChapterMarker()

    def render_is1(self, token):    self.renderS(token)
    def render_imt1(self, token):   self.write(u'</p><h2>' + token.value + u'</h2><p>')
    def render_imt2(self, token):   self.write(u'</p><h3>' + token.value + u'</h3><p>')
    def render_imt3(self, token):   self.write(u'</p><h4>' + token.value + u'</h4><p>')
    def render_ip(self, token):     self.renderP(token)
    def render_iot(self, token):    self.renderQ(token)
    def render_io1(self, token):    self.renderQ2(token)

    def renderFS(self, token):      self.write(u'<span class="rightnotemarker">*</span><span class="rightnote">')
    def renderFE(self, token):      self.write(u'</span>')
    def renderFP(self, token):      self.write(u'<br />')


#
#  Structure
#

header = r"""<!DOCTYPE html>
    <html lang="en">
    <head>
    <title>Open English Bible</title>
    <meta charset='utf-8'>
    <style type="text/css">
    @media all {
        html {font-size: 19px;}
        body {
            padding: 0rem 0em 0rem 0em;
            margin-left:auto;
            margin-right:auto;
            width:100%;
            min-height: 100%;
            position: relative;
        }
        body > * {
            font-size: 100%;
            line-height: 135%;
            text-rendering: optimizeLegibility;
            margin-left: 7rem;
            margin-right: 7rem;
        }
        .chapter{
        	position: absolute;
        	left: 0rem;
        	width: 6rem;
        	text-align: right;
        	font-size: 120%;
        	color: #202020;
        }
        .verse{
        	width: 6rem;
        	text-align: right;
        	font-size: 80%;
        	color: gray;
        }
        .rightnotemarker{
            color: gray;
        }
        .rightnote{
        	position: absolute;
        	right: 0rem;
        	width: 6rem;
        	text-align: left;
        	color: gray;
        	font-size: 80%;
        }
        h1{
        	font-family: 'Verdana', sans-serif;
        	font-size: 180%;
        	color: #202020;
        }
        h2{
        	font-family: 'Verdana', sans-serif;
        	font-size: 140%;
        	color: #202020;
        }
        h3{
        	font-family: 'Verdana', sans-serif;
        	font-size: 120%;
        	color: #202020;
        }
        h4{
        	font-family: 'Verdana', sans-serif;
        	font-size: 100%;
        	color: #202020;
            padding-top:2em;
        }
        h5{
        	font-family: 'Verdana', sans-serif;
        	font-size: 100%;
        	color: #202020;
            padding-top:2em;
        }
        h6{
        	font-family: 'Verdana', sans-serif;
        	font-size: 100%;
        	color: #202020;
            padding-top:0em;
        }
        h7{
        	font-family: 'Verdana', sans-serif;
        	font-size: 100%;
        	color: #202020;
            padding-top:0em;
        }
        p{
        	-webkit-hyphens: auto;
        	-moz-hyphens: auto;
        	-ms-hyphens: auto;
        	-o-hyphens: auto;
        	hyphens: auto;
        	font-family: 'Verdana', sans-serif;
        	color: #202020;
            -moz-font-feature-settings: "liga=1, dlig=1", "onum=1";
            -ms-font-feature-settings: "liga", "dlig","onum";
            -webkit-font-feature-settings: "liga", "dlig","onum";
            -o-font-feature-settings: "liga", "dlig","onum";
            font-feature-settings: "liga", "dlig","onum";
        }
        .align-right {
            align:right;
        }
        .nd { /* Lord */
            font-variant:small-caps;
        }
        .vspacer{
            height:1em;
        }
    }
    @media all and (max-width:800px){html {font-size: 19px;}}
    @media all and (max-width:760px){html {font-size: 18px;}}
    @media all and (max-width:720px){html {font-size: 17px;}}
    @media all and (max-width:680px){html {font-size: 16px;}}
    @media all and (max-width:640px){html {font-size: 14px;}}
    @media all and (max-width:600px){html {font-size: 12px;}}

    /* iPhone 2 - 4 */
    @media only screen
    and (min-device-width : 320px)
    and (max-device-width : 480px)
    and (orientation : portrait) {
        html {font-size: 58px;}
        body {
            margin:0;
            padding:0;
        }
        body > * { margin-left:100px; margin-right:0px; width:100%; }
        .chapter{
        	position: absolute;
        	left: 20px;
        	width: 60px;
        	text-align: left;
        	font-size: 100%;
        	color: green;
        }
        .verse{
        	position: absolute;
        	left: 20px;
        	width: 60px;
        	text-align: left;
        	font-size: 80%;
        	color: green;
        }
        .navbar {
            position:relative;
        }
    }
    </style>
    </head>

    <body>
        """

footer = r"""
        </p></body>
        """

indexPage = header + r"""<h1>Bible</h1>""" + footer
